unit GraphicsWorkUnit;

// In the Project .DPR file, the WorkUnit must be set up as follows
(*
program Project1;

uses
  Forms,
  Unit1 in 'Unit1.pas' {Form1},
  FileWorkUnit in 'FileWorkUnit.pas';
*)

// In the Project->Options->Directories/Conditionals,
// ... Search Path = X:\TeleFlow\CommonUnits

(*  ------ Some examples ------

      //Current.Picture := nil;
      //Current.Picture.LoadFromFile('coolphone.png');

      //PNGDraw(PNGFile,0,0);
      Current.Picture := nil;
      Current.Picture.Assign(PNGFile);

      //GraphicScale(Image3.Picture.Bitmap ,Image4.Picture.Bitmap );
      Image4.Picture.Bitmap.PixelFormat := pf24Bit;

      Image4.Picture.Bitmap.Canvas.Brush.Color := clBlue;
      Image4.Picture.Bitmap.Canvas.FillRect(Rect(0,0,100,100));
      GraphicScaleDown(Image3.Picture.Bitmap ,Image4.Picture.Bitmap, Image4.Width, Image4.Height, 1);


      with TPNGObject(Current.Picture.Graphic),Header do
      begin
           if ColorType = COLOR_RGBALPHA then
           begin
                // Resize
                GraphicScalePngToBit(TPNGObject(Current.Picture.Graphic),
                                     Form1.Color,
                                     Smaller.Picture.BitMap,
                                     Smaller.Width,
                                     Smaller.Height,
                                     1);
           end;
           mText := 'Color type: ' + ColorText[ColorType];
           mText := 'Bit depth: ' + inttostr(BitDepth);
           mText := 'Width: ' + inttostr(Width);
           mText := 'Height: ' + inttostr(Height);
      end;

      with Current.Picture.Graphic as TPNGObject do
      begin
           mInfo :=  PartialTransparency;
           if mInfo then Form1.Color := clWhite;
           //Draw(imgStTest.Picture.Bitmap.Canvas,Rect(1,1,150,150));
      end;

*)



interface

uses
  ShellAPI,Messages,SysUtils,Classes,Windows,Graphics,pngimage;

type
  PPalEntriesArray = ^TPalEntriesArray; {for palette re-construction}
  TPalEntriesArray = array[0..0] of TPaletteEntry;

    procedure GraphicScalePngToBit(pOriginalPng: TPNGObject; pBackGroundColor: TColor; pResizeBit: TBitmap; pWidth,pHeight,pBorder: Integer);
    procedure GraphicScaleDown(pOriginalBit,pResizeBit: TBitmap; pWidth,pHeight,pBorder: Integer);
    procedure GraphicCreateAsMask(pMaskBit: TBitmap);
    function  GraphicBrightnessAdjust(pColor: TColor; pPercent: Integer): TColor;
    procedure GraphicsPrintBltTBitmapAsDib(DestDc : hdc;   {Handle of where to blt}
                                           x : word;       {Bit at x}
                                           y : word;       {Blt at y}
                                           Width : word;   {Width to stretch}
                                           Height : word;  {Height to stretch}
                                           bm : TBitmap);  {the TBitmap to Blt}
    function GraphicsCaptureScreen(pReduceColorDepth: Boolean): TBitmap;
    procedure GraphicsBitToPng(pGraphicFileName: String; pGraphicBitmap: TBitmap);
    procedure GraphicsGreyScale(Bitmap: TBitmap; Brightness, Contrast: Integer; Transparent: Boolean);

implementation

//var
//  GlobErrorFlag: Boolean;


// Capture screen shot
// eg:
// // Capture screen to file
// GraphicsBitToPng('c:\bob.png',GraphicsCaptureScreen(True));
function GraphicsCaptureScreen(pReduceColorDepth: Boolean): TBitmap;
var
   DC : HDC;
   ABitmap:TBitmap;
begin
     DC := GetDC (GetDesktopWindow);
     ABitmap:=TBitmap.Create;
     try
       ABitmap.Transparent := False;
       ABitmap.Width  := GetDeviceCaps (DC, HORZRES);
       ABitmap.Height := GetDeviceCaps (DC, VERTRES);
       BitBlt(ABitmap.Canvas.Handle, 0, 0, ABitmap.Width,
              ABitmap.Height,DC, 0, 0, SRCCOPY);
     finally
       ReleaseDC (GetDesktopWindow, DC);
     end;

     // Reduce color depth to save space
     if pReduceColorDepth then
     begin
          // (pfDevice, pf1bit, pf4bit, pf8bit, pf15bit, pf16bit, pf24bit, pf32bit, pfCustom);
          ABitmap.PixelFormat := pf8bit;
     end;

     Result:=ABitmap;
end;  //GraphicsCaptureScreen


// Save a BitMap as a Png File
procedure GraphicsBitToPng(pGraphicFileName: String; pGraphicBitmap: TBitmap);
var
   // PNG graphics
   pngImage: TPNGObject;
begin
     // Get the Png graphic
     pngImage  := TPNGObject.Create;
     with pngImage do
     begin
          try
             Transparent := False;
             Assign(pGraphicBitmap);

             //CompressionLevel := 9;  // A waste of time... doesnt compress much more
             //Width
             //Height
             // Save it
             SaveToFile(pGraphicFileName);

          finally
             Free;
          end;
     end;
end;  //GraphicsBitToPng


function GraphicBrightnessAdjust(pColor: TColor; pPercent: Integer): TColor;
var
   mRed,
   mGreen,
   mBlue: Integer;
begin
     // Get the colours
     mRed   := Round(pColor and $000000FF);
     mGreen := Round((pColor and $0000FF00) / 256);
     mBlue  := Round((pColor and $00FF0000) / (256 * 256));


     // Adjust them up towards white .. or down to black
     if pPercent >  100 then pPercent  :=  100;
     if pPercent < -100 then pPercent  := -100;

     mRed   := mRed   + Round((255 - mRed   ) * pPercent / 100);
     mGreen := mGreen + Round((255 - mGreen ) * pPercent / 100);
     mBlue  := mBlue  + Round((255 - mBlue  ) * pPercent / 100);

     Result :=(mBlue shl 16)+
              (mGreen shl 8)+
              (mRed);
end; //GraphicBrightnessAdjust


// Build a mask from an image
procedure GraphicCreateAsMask(pMaskBit: TBitmap);
var
   mCol1,
   mCol2,
   mCol3,
   mX,
   mY,
   mWidth,
   mHeight: Integer;

   RowOut: PByteArray;
begin
     mWidth      := pMaskBit.Width;
     mHeight     := pMaskBit.Height;

     // Efficiently scan only as much of the graphic as needed.
     for mY := 0 to (mHeight -1)  do
     begin
          // Any color other than white.... turn to black
          RowOut := pMaskBit.Scanline[mY];
          mCol1  := RowOut[0];
          mCol2  := RowOut[1];
          mCol3  := RowOut[2];

          for mX := 0 to (mWidth -1) do
          begin
               If (RowOut[mX*3]   <> mCol1) or  // $FF
                  (RowOut[mX*3+1] <> mCol2) or
                  (RowOut[mX*3+2] <> mCol3) then
               begin
                    RowOut[mX*3]   := $00;  // Black
                    RowOut[mX*3+1] := $00;
                    RowOut[mX*3+2] := $00;
               end
               else
               begin
                    RowOut[mX*3]   := $FF;  // White
                    RowOut[mX*3+1] := $FF;
                    RowOut[mX*3+2] := $FF;
               end;
          end;
     end;
end;  //GraphicMaskCreate


// Create an image to do a quick draw from
// P1: Original TBitmap
// P2: New, Resized TBitmap
// P3: Resize Width
// P4: Resize Height
// P5: Border width (generally 1)
procedure GraphicScaleDown(pOriginalBit,pResizeBit: TBitmap; pWidth,pHeight,pBorder: Integer);
var
   WorkSmallBit,
   WorkBit:  TBitmap;  // used to make all bit maps same sized

   mPixelLeft,
   mPixelBottom,
   mCol1,
   mCol2,
   mCol3,
   mW,
   mH: Integer;

   SerChar:  String[3];
   SBit: Array[0..12] of Char;
   SCha: Array[0..12] of Char;

   // Reduction vars
   RowIn,
   RowOut: PByteArray;

   x1,
   x2,
   y1,
   y2,
   ix,
   iy,
   total_red,
   total_green,
   total_blue,
   to_y,
   to_x,
   from_x1,
   from_y1,
   from_x2,
   from_y2,
   to_x1,
   to_y1,
   to_x2,
   to_y2: Integer;

   ratio,
   xScale,
   yScale: real;

   pixCol: TColor;

   mBrush1,
   mBrush2: Integer;
begin
// 1) Get StepImage from resource
//   - Find transparent colour
// 2) Create a Work bitmap
//   - Draw the StepImage
// 3) Create the mask image
// 4) Create a mask of the Work bit by affecting colours
// 5) Add the images to image lists
// 6) Now create the small graphic
// 7) Resize

     // Get the Transparent colour (Bottom left corner)
     RowOut := pOriginalBit.Scanline[pOriginalBit.Height - 1];
     mCol1  := RowOut[0];
     mCol2  := RowOut[1];
     mCol3  := RowOut[2];


     // Create a Work bitmap
     WorkBit := TBitmap.Create;
     // Create the small bitmap
     WorkSmallBit := TBitmap.Create;

     try
        with WorkBit do
        begin
             Width       := pOriginalBit.Width;
             Height      := pOriginalBit.Height;
             Transparent := False;

             // Full Colour
             PixelFormat := pf24Bit;

             mBrush1     := (256*256*mCol1) + (256*mCol2) + (mCol3);
             Canvas.Brush.Color := mBrush1;
             Canvas.FillRect(Rect(0,0,Width,Height));

             mW := Round(Width / 2)  - Round(pOriginalBit.Width / 2);
             mH := Round(Height / 2) - Round(pOriginalBit.Height / 2);
             Canvas.Draw(mW, mH, pOriginalBit);

             // Set again to prevent an error
             Width       := pOriginalBit.Width;
             Height      := pOriginalBit.Height;

             // Full Colour
             PixelFormat := pf24Bit;

             Canvas.Draw(0, 0, pOriginalBit);

             mBrush1     := Canvas.Pixels[0,pOriginalBit.Height - 1];

             // Set again to prevent an error
             Width       := pOriginalBit.Width;
             Height      := pOriginalBit.Height;
        end;



        // 6) Now create the small graphic
        // ===============================
        // Get the Transparent colour (Bottom left corner)
        //RowOut := pResizeBit.Scanline[pResizeBit.Height - 1];
        //mCol1  := RowOut[0];
        //mCol2  := RowOut[1];
        //mCol3  := RowOut[2];

        with WorkSmallBit do
        begin
             Width       := pWidth;
             Height      := pHeight;
             Transparent := False;


             //mBrush2     := (256*256*mCol1) + (256*mCol2) + (mCol3);
             //Canvas.Brush.Color := mBrush2;
             //Canvas.FillRect(Rect(0,0,Width,Height));

             // Full Colour
             PixelFormat := pf24Bit;

             Width       := pWidth;
             Height      := pHeight;
        end;


        // 7) Resize
        // =========
        from_x1 :=  0;
        from_y1 :=  0;
        from_x2 :=  WorkBit.Width - 1;
        from_y2 :=  WorkBit.Height - 1 ;
        to_x1   :=  pBorder;
        to_y1   :=  pBorder;
        to_x2   :=  WorkSmallBit.Width - 1 - pBorder;
        to_y2   :=  WorkSmallBit.Height - 1 - pBorder;


        // Must go smaller
        if to_x2 > from_x2 then to_x2 := from_x2 - 1;
        if to_y2 > from_y2 then to_y2 := from_y2 - 1;

        // Compute the scaling parameters. This is useful if
        // the image is not being scaled proportionally.
        xScale := (to_x2 - to_x1 + 1) / (from_x2 - from_x1);
        yScale := (to_y2 - to_y1 + 1) / (from_y2 - from_y1);


        // Perform the reduction.
        for to_y := to_y1 to to_y2 do
        begin
          y1 := Round( (to_y - to_y1) / yScale + from_y1);            // Trunc
          y2 := Round( (to_y + 1 - to_y1) / yScale + from_y1 - (1));  // Trunc

          RowIn := WorkSmallBit.Scanline[to_y];

          for to_x := to_x1 to to_x2 do
          begin

            x1 := Round( (to_x - to_x1) / xScale + from_x1);            // Trunc
            x2 := Round( (to_x + 1 - to_x1) / xScale + from_x1 - (1));  // Trunc

            // Average the values in from_canvas within
            // the box (x1, y1) - (x2, y2).
            total_red   := 0;
            total_green := 0;
            total_blue  := 0;

            for iy := y1 to y2 do
            begin

              RowOut := WorkBit.Scanline[iy];

              for ix := x1 to x2 do
              begin
                // Separate a Color (Pixels[][]) into red, green, and blue components.
                // ScanLine is much faster

                //pixCol = from_canvas->Pixels[ix][iy];  // Very Slow
                //pixCol := TColor(LinePtr[iy][ix]);
                pixCol := TColor(RowOut[ix]);

                total_red   := total_red   + RowOut[ix *3 + 0]; //Round((pixCol) mod 256);          // Mod
                total_green := total_green + RowOut[ix *3 + 1]; //Round(Round(pixCol) /  256) mod 256; // Div // Mod      Scanline
                total_blue  := total_blue  + RowOut[ix *3 + 2]; //Round((pixCol) /   65536);        // DIv

              end;
            end;

            // Weight the amount of colour to draw based on area sampled
            ratio := 1 / (x2 - x1 + 1) / (y2 - y1 + 1);

            RowIn[to_x * 3+0] := Round(total_red   * ratio);
            RowIn[to_x * 3+1] := Round(total_green * ratio);
            RowIn[to_x * 3+2] := Round(total_blue  * ratio);

          end; // End for to_x
        end; // End for to_y

        // And make transparent if the border is one or more
        if pBorder > 0 then
        begin
             WorkSmallBit.Canvas.Brush.Color := mBrush1;
             WorkSmallBit.Canvas.FloodFill(0,0,WorkSmallBit.Canvas.Pixels[0,0],fsSurface);
        end;

        // Assign the new graphic...
        pResizeBit.Assign(WorkSmallBit);

     finally
        // Destroy the graphic
        WorkSmallBit.Free;
        WorkBit.Free;
     end;
end; //GraphicScaleDown




// Create an image to do a quick draw from, and include alpha channel
//procedure TForm1.GraphicScaleDownPngAlpha(pOriginalPng,pResizePng: TPNGObject; pWidth,pHeight,pBorder: Integer);
procedure GraphicScalePngToBit(pOriginalPng: TPNGObject; pBackGroundColor: TColor; pResizeBit: TBitmap; pWidth,pHeight,pBorder: Integer);
var
   WorkSmallBit,
   WorkBit:  TBitmap;  // used to make all bit maps same sized

   mPixelLeft,
   mPixelBottom,
   mCol1,
   mCol2,
   mCol3,
   mW,
   mH: Integer;

   SerChar:  String[3];
   SBit: Array[0..12] of Char;
   SCha: Array[0..12] of Char;

   // Reduction vars
   RowIn,
   RowOut: PByteArray;

   x1,
   x2,
   y1,
   y2,
   ix,
   iy,
   total_red,
   total_green,
   total_blue,
   to_y,
   to_x,
   from_x1,
   from_y1,
   from_x2,
   from_y2,
   to_x1,
   to_y1,
   to_x2,
   to_y2: Integer;

   ratio,
   xScale,
   yScale: real;

   pixCol: TColor;

   mBrush1,
   mBrush2: Integer;
begin
     // Get the Transparent colour (Bottom left corner)
     RowOut := pOriginalPng.Scanline[pOriginalPng.Height - 1];
     mCol1  := RowOut[0];
     mCol2  := RowOut[1];
     mCol3  := RowOut[2];


     // Create a Work bitmap
     WorkBit := TBitmap.Create;
     // Create the small bitmap
     WorkSmallBit := TBitmap.Create;
     try
        with WorkBit do
        begin
             Width       := pOriginalPng.Width;
             Height      := pOriginalPng.Height;
             Transparent := False;

             // Full Colour
             PixelFormat := pf24Bit;

             //mBrush1     := (256*256*mCol1) + (256*mCol2) + (mCol3);
             mBrush1     := pBackGroundColor;
             Canvas.Pen.Color   := mBrush1;
             Canvas.Brush.Color := mBrush1;
             Canvas.FillRect(Rect(0,0,Width,Height));

             mW := Round(Width / 2)  - Round(pOriginalPng.Width / 2);
             mH := Round(Height / 2) - Round(pOriginalPng.Height / 2);
             Canvas.Draw(mW, mH, pOriginalPng);

             // Set again to prevent an error
             Width       := pOriginalPng.Width;
             Height      := pOriginalPng.Height;

             // Full Colour
             PixelFormat := pf24Bit;

             Canvas.Draw(0, 0, pOriginalPng);

             mBrush1     := Canvas.Pixels[0,pOriginalPng.Height - 1];

             // Set again to prevent an error
             Width       := pOriginalPng.Width;
             Height      := pOriginalPng.Height;
        end;


        // 6) Now create the small graphic
        // ===============================
        with WorkSmallBit do
        begin
             Width       := pWidth;
             Height      := pHeight;
             Transparent := False;

             // Full Colour
             PixelFormat := pf24Bit;

             mBrush1     := pBackGroundColor;
             Canvas.Pen.Color   := mBrush1;
             Canvas.Brush.Color := mBrush1;
             Canvas.FillRect(Rect(0,0,Width,Height));

             Width       := pWidth;
             Height      := pHeight;
        end;


        // 7) Resize
        // =========
        from_x1 :=  0;
        from_y1 :=  0;
        from_x2 :=  WorkBit.Width - 1;
        from_y2 :=  WorkBit.Height - 1 ;
        to_x1   :=  pBorder;
        to_y1   :=  pBorder;
        to_x2   :=  WorkSmallBit.Width - 1 - pBorder;
        to_y2   :=  WorkSmallBit.Height - 1 - pBorder;


        // Must go smaller
        if to_x2 > from_x2 then to_x2 := from_x2 - 1;
        if to_y2 > from_y2 then to_y2 := from_y2 - 1;

        // Compute the scaling parameters. This is useful if
        // the image is not being scaled proportionally.
        xScale := (to_x2 - to_x1 + 1) / (from_x2 - from_x1);
        yScale := (to_y2 - to_y1 + 1) / (from_y2 - from_y1);


        // Perform the reduction.
        for to_y := to_y1 to to_y2 do
        begin
          y1 := Round( (to_y - to_y1) / yScale + from_y1);            // Trunc
          y2 := Round( (to_y + 1 - to_y1) / yScale + from_y1 - (1));  // Trunc

          RowIn := WorkSmallBit.Scanline[to_y];

          for to_x := to_x1 to to_x2 do
          begin

            x1 := Round( (to_x - to_x1) / xScale + from_x1);            // Trunc
            x2 := Round( (to_x + 1 - to_x1) / xScale + from_x1 - (1));  // Trunc

            // Average the values in from_canvas within
            // the box (x1, y1) - (x2, y2).
            total_red   := 0;
            total_green := 0;
            total_blue  := 0;

            for iy := y1 to y2 do
            begin

              RowOut := WorkBit.Scanline[iy];

              for ix := x1 to x2 do
              begin
                // Separate a Color (Pixels[][]) into red, green, and blue components.
                // ScanLine is much faster

                //pixCol = from_canvas->Pixels[ix][iy];  // Very Slow
                //pixCol := TColor(LinePtr[iy][ix]);
                pixCol := TColor(RowOut[ix]);

                total_red   := total_red   + RowOut[ix *3 + 0]; //Round((pixCol) mod 256);          // Mod
                total_green := total_green + RowOut[ix *3 + 1]; //Round(Round(pixCol) /  256) mod 256; // Div // Mod      Scanline
                total_blue  := total_blue  + RowOut[ix *3 + 2]; //Round((pixCol) /   65536);        // DIv

              end;
            end;

            // Weight the amount of colour to draw based on area sampled
            ratio := 1 / (x2 - x1 + 1) / (y2 - y1 + 1);

            RowIn[to_x * 3+0] := Round(total_red   * ratio);
            RowIn[to_x * 3+1] := Round(total_green * ratio);
            RowIn[to_x * 3+2] := Round(total_blue  * ratio);

          end; // End for to_x
        end; // End for to_y

        // Assign the new graphic...
        pResizeBit.Assign(WorkSmallBit);

     finally
        // Destroy the graphic
        WorkSmallBit.Free;
        WorkBit.Free;
     end;

end; //GraphicScalePngToBit



// Print Image to printer
(*
     // Print it
     Printer.BeginDoc;
     GraphicsPrintBltTBitmapAsDib(Printer.Canvas.Handle,
                                  100,
                                  100,
                                  Printer.PageWidth - 100,
                                  Printer.PageHeight - 100,
                                  mBitMap1); // BuildForm.FlowTablet.FlowImage);
     Printer.EndDoc;
*)
procedure GraphicsPrintBltTBitmapAsDib(DestDc : hdc;   {Handle of where to blt}
                                       x : word;       {Bit at x}
                                       y : word;       {Blt at y}
                                       Width : word;   {Width to stretch}
                                       Height : word;  {Height to stretch}
                                       bm : TBitmap);  {the TBitmap to Blt}
var
  OriginalWidth :LongInt;               {width of BM}
  dc : hdc;                             {screen dc}
  IsPaletteDevice : bool;               {if the device uses palettes}
  IsDestPaletteDevice : bool;           {if the device uses palettes}
  BitmapInfoSize : integer;             {sizeof the bitmapinfoheader}
  lpBitmapInfo : PBitmapInfo;           {the bitmap info header}
  hBm : hBitmap;                        {handle to the bitmap}
  hPal : hPalette;                      {handle to the palette}
  OldPal : hPalette;                    {temp palette}
  hBits : THandle;                      {handle to the DIB bits}
  pBits : pointer;                      {pointer to the DIB bits}
  lPPalEntriesArray : PPalEntriesArray; {palette entry array}
  NumPalEntries : integer;              {number of palette entries}
  i : integer;                          {looping variable}
begin
(*
   If range checking is on - lets turn it off for now
   we will remember if range checking was on by defining
   a define called CKRANGE if range checking is on.
   We do this to access array members past the arrays
   defined index range without causing a range check
   error at runtime. To satisfy the compiler, we must
   also access the indexes with a variable. ie: if we
   have an array defined as a: array[0..0] of byte,
   and an integer i, we can now access a[3] by setting
   i := 3; and then accessing a[i] without error
*)

{$IFOPT R+}
  {$DEFINE CKRANGE}
  {$R-}
{$ENDIF}

     // Save the original width of the bitmap
     OriginalWidth := bm.Width;

     // Get the screen's dc to use since memory dc's are not reliable
     dc              := GetDc(0);
     // Are we a palette device?
     IsPaletteDevice := GetDeviceCaps(dc, RASTERCAPS) and RC_PALETTE = RC_PALETTE;
     // Give back the screen dc
     dc              := ReleaseDc(0, dc);

     // Allocate the BitmapInfo structure
     if IsPaletteDevice then
     begin
          BitmapInfoSize := sizeof(TBitmapInfo) + (sizeof(TRGBQUAD) * 255);
     end
     else
     begin
          BitmapInfoSize := sizeof(TBitmapInfo);
     end;
     GetMem(lpBitmapInfo, BitmapInfoSize);

     // Zero out the BitmapInfo structure
     FillChar(lpBitmapInfo^, BitmapInfoSize, #0);

     // Fill in the BitmapInfo structure
     lpBitmapInfo^.bmiHeader.biSize := sizeof(TBitmapInfoHeader);
     lpBitmapInfo^.bmiHeader.biWidth := OriginalWidth;
     lpBitmapInfo^.bmiHeader.biHeight := bm.Height;
     lpBitmapInfo^.bmiHeader.biPlanes := 1;

     if IsPaletteDevice then
     begin
          lpBitmapInfo^.bmiHeader.biBitCount := 8;
     end
     else
     begin
          lpBitmapInfo^.bmiHeader.biBitCount := 24;
     end;
     lpBitmapInfo^.bmiHeader.biCompression := BI_RGB;
     lpBitmapInfo^.bmiHeader.biSizeImage :=
       ((lpBitmapInfo^.bmiHeader.biWidth *
         longint(lpBitmapInfo^.bmiHeader.biBitCount)) div 8) *
         lpBitmapInfo^.bmiHeader.biHeight;
     lpBitmapInfo^.bmiHeader.biXPelsPerMeter := 0;
     lpBitmapInfo^.bmiHeader.biYPelsPerMeter := 0;
     if IsPaletteDevice then
     begin
          lpBitmapInfo^.bmiHeader.biClrUsed := 256;
          lpBitmapInfo^.bmiHeader.biClrImportant := 256;
     end
     else
     begin
          lpBitmapInfo^.bmiHeader.biClrUsed := 0;
          lpBitmapInfo^.bmiHeader.biClrImportant := 0;
     end;

     // Take ownership of the bitmap handle and palette
     hBm  := bm.ReleaseHandle;
     hPal := bm.ReleasePalette;

     // Get the screen's dc to use since memory dc's are not reliable
     dc := GetDc(0);

     if IsPaletteDevice then
     begin
          // If we are using a palette, it must be
          // selected into the dc during the conversion
          OldPal := SelectPalette(dc, hPal, TRUE);
          // Realize the palette
          RealizePalette(dc);
     end;

     // Tell GetDiBits to fill in the rest of the bitmap info structure
     GetDiBits(dc,
               hBm,
               0,
               lpBitmapInfo^.bmiHeader.biHeight,
               nil,
               TBitmapInfo(lpBitmapInfo^),
               DIB_RGB_COLORS);

     // Allocate memory for the Bits
     hBits := GlobalAlloc(GMEM_MOVEABLE,
                          lpBitmapInfo^.bmiHeader.biSizeImage);
     pBits := GlobalLock(hBits);

     // Get the bits
     GetDiBits(dc,
               hBm,
               0,
               lpBitmapInfo^.bmiHeader.biHeight,
               pBits,
               TBitmapInfo(lpBitmapInfo^),
               DIB_RGB_COLORS);


     if IsPaletteDevice then
     begin
          // Lets fix up the color table for buggy video drivers
          GetMem(lPPalEntriesArray, sizeof(TPaletteEntry) * 256);
          // $IFDEF VER100
            NumPalEntries := GetPaletteEntries(hPal,
                                               0,
                                               256,
                                               lPPalEntriesArray^);
          // $ELSE
            NumPalEntries := GetSystemPaletteEntries(dc,
                                                     0,
                                                     256,
                                                     lPPalEntriesArray^);
          // $ENDIF
          for i := 0 to (NumPalEntries - 1) do
          begin
               lpBitmapInfo^.bmiColors[i].rgbRed :=
                 lPPalEntriesArray^[i].peRed;
               lpBitmapInfo^.bmiColors[i].rgbGreen :=
                 lPPalEntriesArray^[i].peGreen;
               lpBitmapInfo^.bmiColors[i].rgbBlue :=
                 lPPalEntriesArray^[i].peBlue;
          end;
          FreeMem(lPPalEntriesArray, sizeof(TPaletteEntry) * 256);
     end;

     if IsPaletteDevice then
     begin
          // Select the old palette back in
          SelectPalette(dc, OldPal, TRUE);
          // Realize the old palette
          RealizePalette(dc);
     end;

     // Give back the screen dc
     dc := ReleaseDc(0, dc);

     // Is the Dest dc a palette device?
     IsDestPaletteDevice :=
       GetDeviceCaps(DestDc, RASTERCAPS) and RC_PALETTE = RC_PALETTE;


     if IsPaletteDevice then
     begin
          // If we are using a palette, it must be
          // selected into the dc during the conversion
          OldPal := SelectPalette(DestDc, hPal, TRUE);
          // Realize the palette
          RealizePalette(DestDc);
     end;

     // Do the blt stretch
     StretchDiBits(DestDc,
                   x,
                   y,
                   Width,
                   Height,
                   0,
                   0,
                   OriginalWidth,
                   lpBitmapInfo^.bmiHeader.biHeight,
                   pBits,
                   lpBitmapInfo^,
                   DIB_RGB_COLORS,
                   SrcCopy);

     if IsDestPaletteDevice then
     begin
          // Select the old palette back in
          SelectPalette(DestDc, OldPal, TRUE);
          // Realize the old palette
          RealizePalette(DestDc);
     end;

     // De-Allocate the Dib Bits
     GlobalUnLock(hBits);
     GlobalFree(hBits);

     // De-Allocate the BitmapInfo
     FreeMem(lpBitmapInfo, BitmapInfoSize);

     // Set the ownership of the bimap handles back to the bitmap
     bm.Handle := hBm;
     bm.Palette := hPal;

     // Turn range checking back on if it was on when we started
{$IFDEF CKRANGE}
  {$UNDEF CKRANGE}
  {$R+}
{$ENDIF}
end; // BltTBitmapAsDib



// Grey out bitmaps, and can brighten and change the contrast of.
// Note: @@@ Transparent needs to be supported
(*
      SpeedButton2.Glyph := Image3.Picture.Bitmap;
      GraphicsGrayscale(SpeedButton2.Glyph,84,0);
*)
procedure GraphicsGreyScale(Bitmap: TBitmap; Brightness, Contrast: Integer; Transparent: Boolean);
type
  pRGBLine = ^TRGBLine;
  TRGBLine = Array[word] of TRGBTriple;
  pRGBQuadLine = ^TRGBQuadLine;
  TRGBQuadLine = Array[word] of TRGBQuad;
var
   Line    : Pointer;
   i, j    : Integer;
   palCount: Word;
   MaxPal  : TMaxLogPalette;
   pf24    : boolean;
   mColor  : Integer;
begin
     case Bitmap.PixelFormat of
     pf1bit, pf4bit, pf8bit:  {Palette}
     begin
         {Retrieve the number of palette entries}
         GetObject(Bitmap.Palette, sizeof(word), @palCount);
         MaxPal.palVersion := $0300;
         MaxPal.palNumEntries := PalCount;
         GetPaletteEntries(Bitmap.Palette, 0, PalCount,MaxPal.palpalentry);

         for j := 0 to PalCount - 1 do
         begin
              with MaxPal.palPalEntry[j] do
              begin
                   peRed := (peRed + peGreen + peBlue) div 3;
                   peGreen := peRed;
                   peBlue := peRed;
              end;
         end;

         Bitmap.Palette := CreatePalette(PLogPalette(@MaxPal)^);

     end;

     pf15bit, pf16bit:
     begin
          raise Exception.Create('15bit and 16bit bitmap grayscale conversion not supported');
     end;

     pf24bit, pf32bit:  {24 bit}
     begin
          pf24 := (Bitmap.PixelFormat = pf24bit);

          for j := 0 to Bitmap.Height - 1 do
          begin
               Line := Bitmap.Scanline[j];
               for i := 0 to Bitmap.Width - 1 do
               begin
                    if pf24 then
                    begin
                         with pRGBLine(Line)[i] do
                         begin
                              mColor    := (rgbtRed + rgbtGreen + rgbtBlue) div 3;
                              // Brightness
                              mColor := mColor + Brightness;
                              // Contrast
                              if mColor > 127 then mColor := mColor + Contrast;
                              if mColor < 127 then mColor := mColor  - Contrast;

                              if mColor > 255 then mColor := 255;
                              if mColor < 0 then mColor := 0;
                              rgbtRed   := mColor;
                              rgbtgreen := mColor;
                              rgbtblue  := mColor;
                         end;
                    end
                    else
                    begin
                         with pRGBQuadLine(Line)[i] do
                         begin
                              mColor    := (rgbRed + rgbGreen + rgbBlue) div 3;
                              // Brightness
                              mColor := mColor + Brightness;
                              // Contrast

                              if mColor > 255 then mColor := 255;
                              if mColor < 0 then mColor := 0;
                              rgbRed    := mColor;
                              rgbgreen  := mColor;
                              rgbblue   := mColor;
                         end;
                    end;
               end; // for
          end; // for j
     end; // 24 bit
     end; // Case
end;  // Grayscale


end.