// ---------INCLUDE FILE-----------------------------------------
// -- Included inside TFSentinel
// -- Included inside TFCleanLogs
// --
const
   // Application information
   appName               = 'TFCleanupLogs';
   appVersion            = '0.90';

   // Types (commands)
   cMaxDays              = 'MaxDays';
   cMinGB                = 'MinGB';

   // Defaults
   cValMaxDays           = 14;
   cValMinGB             = 2;


   // Parameter
   cColFCode             = 1;
   cColFVal              = 2;

   // Delimiter
   cD                    = '~';

var

   s1                    : String;
   s2                    : String;
   mError                : String;
   mLoop                 : Integer;
   mRecCount             : Integer;

   //
   mDate                 : String;
   mMaxDays              : Integer;
   mMinGB                : Integer;

   //
   mSupress              : Boolean;
   mHeadingDisplayed     : Boolean;
   mShowHelp             : Boolean;
   mContinue             : Boolean;

   mDrive                : String;
   mLogFile              : String;
   mFreeGB               : Integer;
   mLogDirectory         : String;

   // Read in the parameters
   mParmArray            : array [1..200, 1..4] of String;  // For paramters .. see cCol??? values
   mFCode                : String;
   mFVal                 : String;

   // Scan for files
   mFileList             : TStringList;
   mFileName             : String;
   mCountLogs            : Integer;
   mCountDirs            : Integer;

   /////////////////////////////////////
   procedure DisplayHeading;
   begin
        if not mSupress then
        begin
             mHeadingDisplayed := True;

             writeln(appName);
             writeln('(c) 1994 - 2007 enGenic - Version '+appVersion+' ');
             // Put your name above etc.
             writeln('/? for Help.');

             // Only if help was asked for
             if mShowHelp then
             begin
                  writeln('');
                  writeln('> *** WARNING ***');
                  writeln('> Deletes TeleFlow Server Log and SR log files and recordings');
                  writeln('');
                  writeln('Usage:');
                  writeln('  '+appName+' -f value [-f value] [options]');
                  writeln('');
                  writeln('Where -f is');
                  writeln('  -MaxDays n   (Keep log files for a maximum n calendar days)');
                  writeln('               (Note: '+IntToStr(cValMaxDays)+' Days is assumed)');
                  writeln('  -MinGB   n   (Override MaxDays, and continue removing until n GB is free');
                  writeln('               (Note: '+IntToStr(cValMinGB)+' GB is assumed)');
                  writeln('  [options] -S (Supress '+appName+' title and information)');
                  writeln('  [options] -C (Force Cleaner when no -f options are provided)');
                  writeln('  /? for this help page');
                  writeln('  OutPutFile will be appended if it already exists');
                  writeln('');
                  writeln('Examples:');
                  writeln('   '+appName+' -MaxDays 31 -MinGB 3');
                  writeln('   '+appName+' -C');
             end;
        end;
   end; // DisplayHeading


   // Read the TeleFlow Registry
   procedure ReadTeleFlowRegistry;
   var
   {$I  TeleFlowRegistryRead.pas}
   end;


   // Output to writeln
   procedure Output(pLine: String);
   begin
        // Console mode only...
        if not cEmbedded then
        begin
             try
                if not mSupress then
                begin
                     writeln(pLine);
                end;
             except
             end;
        end;
   end; // Output


   // Read the TeleFlow Registry
   function FindDriveSpaceFreeMB(pDrive: String): Integer;
   var
      // Gather data
      mLoop           : Integer;
      mDrive          : String;
      mDriveList      : String;
      mNumDrive       : Integer;
      mFree           : Integer;
      //mSize           : Integer;
      //mPercent        : Integer;
   begin
        result := -1;
        // Drive space
        ////////////////
        try
           mDriveList    := UpperCase(Copy(pDrive,1,1)); //GetLocalHardDrives;
           for mLoop := 1 to length(mDriveList) do
           begin
                mDrive := Copy(mDriveList,mLoop,1);
                if (mDrive >= 'A') and (mDrive <= 'Z') then
                begin
                     mNumDrive := Pos(mDrive,'ABCDEFGHIJKLMNOPQRSTUVWXYZ');
                     //mSize     := Round(DiskSize(mNumDrive) / (1024 * 1024));
                     mFree     := Round(DiskFree(mNumDrive) / (1024 * 1024));

                     //mResult := mResult + mDrive + ':' + IntToStr(mFree) + ' MB   ';
                     //mPercent  := Round(mFree / mSize * 100);

                     // Get the GB
                     result    := Round(mFree / 1000);
                end;
           end; // drivespace
        except
        end;
   end;



///////////////////////////////////////////
///////////////////////////////////////////
// PROGRAM BEGINS HERE
///////////////////////////////////////////
///////////////////////////////////////////

begin
  try
     mContinue           := True;
     mSupress            := False;
     mShowHelp           := False;
     mHeadingDisplayed   := False;

     // Default values
     mMaxDays            := cValMaxDays;
     mMinGB              := cValMinGB;

     // Log file
     if not cEmbedded then
     begin
          // Stand alone program
          mLogFile := ChangeFileExt(ParamStr(0),'.log');

          // Quick check for help or not enough values
          s1  := ParamStr(1);
          s2  := ParamStr(2);
     end
     else
     begin
          // Embedded
          mLogFile := IncludeTrailingPathDelimiter(Glob_Reg_LogDir)+
                      (ChangeFileExt(ExtractFileName(ParamStr(0)),'')+'_'+
                       FormatDateTime('yyyy"-"mm"-"dd"',Now)+'.log');

          s1  := '-C';  // Force the options

          LogFileOutput(mLogFile, '-- Removing log files with parameters:  '+
                                  cMaxDays+': '+IntToStr(cValMaxDays)+'   '+
                                  cMinGB+  ': '+IntToStr(cValMinGB));
     end;

     // If a Force is being done, then allow it to go forward
     if (s1 = '-C') then
     begin
          mContinue := True;
     end
     // See if the
     else if (s2 = '') or (s1 = '/h') or (s1 = '/?') or (s1 = '-h') then
     begin
          mShowHelp := True;
          mContinue := False;
          mError    := 'Help';
     end;

     // Is TeleFlow installed?
     ReadTeleFlowRegistry;
     mLogDirectory  := GlobRegLOGDIRECTORY;

     //@@@ DEBUG ONLY
     //mLogDirectory  := 'E:\Junk';
     //Glob_TeleFlowInstalled := True; //

     if not Glob_TeleFlowInstalled then
     begin
          mError := 'TeleFlow Server not installed';
          mContinue := False;
     end
     else
     begin
          // If the directory exists, then continue
          if DirectoryExists(mLogDirectory) then
          begin

          end
          else
          begin
               mError := 'Unable to locate log directory: '+ mLogDirectory;
               mContinue := False;
          end;
     end;



     // Lets check that the parameters are good.
     if mContinue then
     begin
          mRecCount := 0;

          // Load all parameters in the mParmArray
          // - Must be even number of params, unless [options or help is thrown]
          for mLoop := 1 to Trunc(ParamCount / 2) do
          begin
               mFCode   := UpperCase(ParamStr(mLoop*2 - 1));
               mFVal    := ParamStr(mLoop*2);

               // Did the user pass a valid code?
               if Pos(UpperCase(Trim(Copy(mFCode,2,99))+cD),
                             (cD + UpperCase(cMaxDays)   + cD +
                              cD + UpperCase(cMinGB)     + cD +
                              cD)) = 0 then
               begin
                    mContinue := False;
                    mError    := 'Invalid -f Code: '+mFCode;
                    break;
               end;

               // Next record
               inc(mRecCount);

               // If we passed muster then
               mParmArray[ mRecCount, cColFCode    ] := UpperCase(Trim(Copy(mFCode,2,99)));
               mParmArray[ mRecCount, cColFVal     ] := mFVal;

               // Assign the new values
               if UpperCase(mParmArray[ mRecCount, cColFCode]) = UpperCase(cMaxDays) then
               begin
                    mMaxDays := StrToIntDef(mFVal, cValMaxDays);
               end
               else
               if UpperCase(mParmArray[ mRecCount, cColFCode]) = UpperCase(cMinGB) then
               begin
                    mMinGB   := StrToIntDef(mFVal, cValMinGB);
               end;

          end;

          // Take one off the end


          // Figure out any final options
          mFCode := UpperCase(ParamStr(ParamCount));
          if mFCode = '-S' then
          begin
               // Don't show any messages
               mSupress := True;
          end;
     end
     else
     begin
          // Give the Log the error message
          if cEmbedded then
          begin
                LogFileOutput(mLogFile, '  WARNING: '+mError);
          end;
     end;

     // If the paramters are good
     if mContinue then
     begin
          // See if the user blew it, or asked for help
          if  (s1 = '/h') or (s1 = '/?') or (s1 = '-h') then
          begin
               mContinue := False;
               mShowHelp := True;
               // Move to showing help
          end
          else
          begin
               // REMEMBER:
               // The last record will be created

               // Cycle through, and see if any files are missing
               if mContinue then
               begin

               end;
          end;
     end;


     //////////////////////
     // Main Processing Cycle
     //////////////////////
     // If the paramters are good
     if mContinue then
     begin
          LogFileOutput(mLogFile, '-- TFCleanLogs Started --');

          // Start the output
          if not mSupress then
          begin
               if not mHeadingDisplayed then
               begin
                    mShowHelp := False;
                    DisplayHeading;
               end;
          end;

          Output('Logging to: '+ mLogFile);

          // Create
          mFileList := TStringList.Create;
          try
             // build the file list
             mCountLogs := 0;
             mCountDirs := 0;
             mFileList.Clear;


             // Log files
             ////////////////////////////
             Output('Scanning .Log Files: '+mLogDirectory);
             ScanDirMask('*.Log');
             ScanDirectory(mLogDirectory);
             ScanDirTop;
             for mLoop := 0 to ScanDirCount - 1 do
             begin
                  mFileName := ScanDirFetchNext;
                  mDate := StringReplace(RightCopy( ChangeFileExt(mFileName, ''), 10),'-','/', [rfReplaceAll, rfIgnoreCase]);

                  SetStandardDateTimeFormats;
                  if (StrToDate(mDate)) < (Now - mMaxDays) then
                  begin
                       Output(' -f: '+mFileName);
                       // DELETE
                       DeleteFile(mFileName);
                       LogFileOutput(mLogFile, 'Deleted -f: '+mFileName);
                       inc(mCountLogs);
                  end;
             end;


             // Directories
             ////////////////////////////
             Output('Scanning Directories: '+mLogDirectory);
             ScanDirMask('*.*');
             ScanListSubDirectories(mLogDirectory);
             ScanDirTop;
             for mLoop := 0 to ScanDirCount - 1 do
             begin
                  mFileName := ScanDirFetchNext;
                  mDate := StringReplace(RightCopy( ChangeFileExt(mFileName, ''), 10),'-','/', [rfReplaceAll, rfIgnoreCase]);

                  // Remember the file names (in case we need to free up more later
                  mFileList.Add(mFileName);

                  SetStandardDateTimeFormats;
                  if (StrToDate(mDate)) < (Now - mMaxDays) then
                  begin
                       Output(' -d: '+mFileName);
                       // DELETE
                       TreeDelete(mFileName);
                       LogFileOutput(mLogFile, 'Deleted -d: '+mFileName);
                       inc(mCountDirs);
                  end;
             end;

             // Display results
             Output('Log files deleted:   ' + IntToStr(mCountLogs));
             Output('Directories deleted: ' + IntToStr(mCountDirs));

             LogFileOutput(mLogFile, 'Deleted:  -# Log files: '+IntToStr(mCountLogs));
             LogFileOutput(mLogFile, 'Deleted:  -# Dirs:      '+IntToStr(mCountDirs));

             // Determine if we need to delete more to save ensure we have a min GB level
             mDrive  := Copy(mLogDirectory,1,1);
             mFreeGB := FindDriveSpaceFreeMB(mDrive);
             Output('GB free space at:    ' + IntToStr(mFreeGB));

             // Cycle to delete directories
             if mFreeGB <= mMinGB then
             begin
                  mLoop := 0;
                  while mFreeGB <= mMinGB do
                  begin
                       mFileName := mFileList[mLoop];
                       Output(' -d: ' + mFileName);

                       // DELETE
                       TreeDelete(mFileName);
                       LogFileOutput(mLogFile, 'Deleted -d: '+mFileName);
                       inc(mLoop);

                       mFreeGB := FindDriveSpaceFreeMB(mDrive);
                       Output('GB free space now at: ' + IntToStr(mFreeGB));
                       LogFileOutput(mLogFile, 'GB free space: ' + IntToStr(mFreeGB));
                  end;
             end;

          finally
             mFileList.Free;
          end;

          
          // Note that we are done
          LogFileOutput(mLogFile, '-- TFCleanLogs Ended --');

     end; // if mContinue

     //////////////////////
     // Clean up
     //////////////////////


     //////////////////////
     // Exit
     //////////////////////

     // Set exit code to report errors
     if mContinue = True then
     begin
          ExitCode := 0;  // Good
     end
     else
     begin
          if not mSupress then
          begin
               if not mHeadingDisplayed then
               begin
                    DisplayHeading;
               end;

               if not mShowHelp then
               begin
                    Output('');
                    Output('**ERROR: '+mError);
               end;
          end;
          ExitCode := 1;  // Error
     end;

     if not mSupress then
     begin
          Output('Done.');
     end;

  except

  end
// ---------INCLUDE FILE-----------------------------------------
