// Used to PROCESS HTTP requests

// HTTPServerCommandGetWork(AContext: TIdContext;
//   ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo);

procedure AuthFailed;
begin
     AResponseInfo.ContentText := '<html><head><title>Error</title></head><body><h1>Authentication failed</h1>'#13 +
       'Check the source code to discover the password:<br><ul><li>Search for <b>AuthUsername</b> in <b>Main.pas</b>!</ul></body></html>';
     AResponseInfo.AuthRealm := sauthenticationrealm;
end;

procedure AccessDenied;
begin
     AResponseInfo.ContentText := '<html><head><title>Error</title></head><body><h1>Access denied</h1>'#13 +
       'You do not have sufficient privileges to access this document.</body></html>';
     AResponseInfo.ResponseNo := 403;
end;


// Multipart decoder
// =================
// Uses IdMessageClient,
//      IdMessageCoder,
//      IdMessageCoderMIME;

// INDY 10 - Does this in the ReadBody
// Result := TIdMessageDecoderMIME.Create(Owner);
// -- It will Nil the SourceStream in the Init.  The following will recover the SourceStream

(* For Example, this will break apart the following Multipart MIME file.
<START>
--------------------465178917914CL
Content-Disp: form-data; name="Tel"CLCL
3105551234CL
--------------------465178917914CL
Content-Disp: form-data; name="mp3file"; filename="file1" conent-type: jpgetc CLCL
.bi..n.a.ry.d..a.t.a......a....dfwe.f.af.sd...CL
--------------------465178917914CL
Content-Disp: form-data; name="telfile"; filename="file2" conent-type: text/pain CLCL
Vern was hereCL
--------------------465178917914CL--
<END>
*)
procedure DecodeFormData(pUploadDir: String; const Header: String; ASourceStream: TStream);
var
    MsgEnd     : Boolean;
    Decoder    : TIdMessageDecoder;
    Tmp        : String;
    Dest       : TStream;
    mStream    : TMemoryStream;
    mFileList  : TStringList;
    mMimeList  : String;
    mBound     : String;
begin
    mFileList  := TStringList.Create;

    (* // For Debugging.  I highly recommend "ExamDiff Pro" to review files side by side
    mStream := TMemoryStream.Create;
    mStream.LoadFromStream(ASourceStream);//  (mParams, mStream.Size);
    mStream.SaveToFile('E:\enGenic\Web\upload\file.mime');
    mStream.Free;
    exit;
    *)

    // Reset the position.  Just in case
    ASourceStream.Position := 0;

    MsgEnd  := False;
    Decoder := TIdMessageDecoderMIME.Create(nil);
    try
        TIdMessageDecoderMIME(Decoder).MIMEBoundary := TIdMIMEBoundary.FindBoundary(Header);
        Decoder.SourceStream     := ASourceStream;
        Decoder.FreeSourceStream := False;
        Decoder.ReadLn;
        repeat
            Decoder.ReadHeader;

            case Decoder.PartType of
                mcptUnknown:
                    begin
                         // raise Exception('Unknown form data detected');
                    end;
                mcptText:
                    begin
                        Tmp  := Decoder.Headers.Values['Content-Type'];
                        Dest := TMemoryStream.Create;
                        try
                            Decoder := Decoder.ReadBody(Dest, MsgEnd);
                            if AnsiSameText(Fetch(Tmp, ';'),'multipart/mixed') then
                            begin
                                 DecodeFormData(pUploadDir, Tmp, Dest);
                            end
                            else
                            begin
                                 // use Dest as needed...
                            end;
                        finally
                            FreeAndNil(Dest);
                        end;
                    end;
                mcptAttachment:
                    begin
                        Tmp := ExtractFileName(Decoder.FileName);
                        if Tmp <> '' then
                        begin
                             Tmp := IncludeTrailingPathDelimiter(pUploadDir) + Tmp
                        end
                        else
                        begin
                             Tmp := MakeTempFilename(IncludeTrailingPathDelimiter(pUploadDir));
                        end;


                        Dest := TFileStream.Create(Tmp, fmCreate);
                        try
                           // Add name of file to create
                           mFileList.Add(Tmp);
                           Decoder   := Decoder.ReadBody(Dest, MsgEnd);
                        finally
                           FreeAndNil(Dest);
                        end;

                        // If the MIMEBoundary has not been popped, then there is more to do
                        if (Decoder <> nil) then
                        begin
                             // The ReadBody does an Init, and kills the Stream, so, we need to set up again.
                             TIdMessageDecoderMIME(Decoder).MIMEBoundary := TIdMIMEBoundary.FindBoundary(Header);
                             Decoder.SourceStream     := ASourceStream;
                             Decoder.FreeSourceStream := False;
                        end;
                    end;
            end;
        until (Decoder = nil) or MsgEnd;
    finally
        FreeAndNil(Decoder);

        mMimeList := ChangeFileExt(MakeTempFilename(IncludeTrailingPathDelimiter(pUploadDir)),'.UpList');
        // Create a list of files
        mFileList.SaveToFile(mMimeList);
    end;
end;  // DecodeFormData

const
   cConferenceHtml       = 'conference.html';
   cConferenceAccessHtml = 'conferenceaccess.html';
   cWordVerifyHtml       = 'wordverify.html';
   cWordVerifyAccessHtml = 'wordverifyaccess.html';

var
   mVMSRequest     : Boolean;
   mRequestPort    : Integer;
   mTCPResult      : String;
   mDocExt         : String;
   mDocPath        : String;
   mDocFileName    : String;
   LocalDoc        : String;
   ByteSent        : Cardinal;
   ResultFile      : TFileStream;
   mDataStream     : TMemoryStream;
   mStream         : TMemoryStream;
   mTest           : String;
   mHTMLData       : TStringList;
   mSpecialMode    : Boolean;
   mFile           : String;
   mCommandResult  : String;
   mCommandIP      : String;
   mCode           : Integer;
   mSaveDir        : String;
   mParams         : String;
   mField01        : String;
   mField02        : String;
   mHostByRequest  : String;
   mParameters     : String;
   mSize           : LongInt;
   mStreamString   : String;
   mProcessIt      : Boolean;
   mTeleFlowMode   : Boolean;
   mUploadFile     : String;
   mContentType    : String;
   mCommandCheck   : String;

   //
   mPeerIP         : String;

        // New Decoder system
        Decoder, Temp: TIdMessageDecoder;
        MemStream: TStream;
        IdPostStream, IdMemStream: TIdStream;
        MsgEnd: Boolean;
        Boundary: String;
begin
  mHTMLData := TStringList.Create;
  try
     // Capture the IP
     mCommandIP := TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerIP;



     // Determine if something is communicating to us it TeleFlow Mode
     if  Copy(ARequestInfo.Document,1,length(cTFHeader)) = cTFHeader then
     begin
          // TeleFlow Web Page managment
          ///////////////////////////////

          // eg:   http://localhost:5048/&~TFSendFile~FileName.Wav
          mTeleFlowMode := True;
     end
     else
     begin
          // Standard Web Page managment
          ///////////////////////////////

          // eg:   http://localhost:5048/<?xml version='1.0' encoding='utf-8'?><REQUEST action='Add'><ITEM id='1212' firstname='First' lastname='Last' clientid='12345' clientname='DEMO' /></REQUEST>
          mTeleFlowMode := False;
     end;


     // Get the Peer IP
     mPeerIP := TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerIP;

     // Log the request
     LogEventMessage(Format( 'HTTP COMMAND:  %s %s received from %s:%d',
                            [ARequestInfo.Command, ARequestInfo.Document,
                            TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerIP,
                            TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerPort]),
                     True, mPeerIP);

     // What host string did the requestor use to find this NetHub?
     if Pos((':'+IntToStr(Glob_SSL_ActivePort)),ARequestInfo.Host) > 0 then
     begin
          // SSL
          mHostByRequest      := 'https://'+ARequestInfo.Host;
     end
     else
     begin
          mHostByRequest      := 'http://'+ARequestInfo.Host;
     end;

     // Get the bytes received (SSL is one more)
     Glob_Bytes_Received := Glob_Bytes_Received + Length(mHostByRequest) + Length(ARequestInfo.Document);


     // Are we allowing authorized access only?
     if Glob_Reg_AuthOn and
        ((ARequestInfo.AuthUsername <> 'engenic') or (ARequestInfo.AuthPassword <> 'Gu1ness')) then
     begin
          AuthFailed;
          exit;
     end;


     // Managed user session?
     if Glob_Reg_ManSess then
     begin
          ManageUserSession(HTTPServer, AContext, ARequestInfo, AResponseInfo);
     end;

     //
     if (Pos('/session', LowerCase(ARequestInfo.Document)) = 1) then
     begin
          ServeVirtualFolder(AContext, ARequestInfo, AResponseInfo);
     end
     else
     begin
          // Someone is looking for a document
          inc(GlobTotalRequests);

          //mRequestPort := 81; // TIdIOHandlerSocket(AThread.Connection.IOHandler).Binding.PeerPort;
          mRequestPort := TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.Port;

          // Get information about any file requests
          mDocFileName := ReplaceAllSingleChar(ARequestInfo.Document,'/','\');


          /////////////////////////////////////////////
          // Voice Office Requests
          /////////////////////////////////////////////
          mDocPath := AcquireFileBaseDirectory();

          // Build the document name to request
          LocalDoc := mDocPath + mDocFileName;


          // Special mode are internally created pages.
          mSpecialMode := False;
          mVMSRequest  := False;

          // Are we dealing with a post?
          if AnsiSameText(ARequestInfo.Command, 'POST') then
          begin
               // Only process if we are not in TeleFlow MOde
               if not mTeleFlowMode then
               begin
                    // For example... HTTPSendXML will send a post
                    mStream := TMemoryStream.Create;
                    try
                       // Any information coming from the post?
                       mStreamString := '';

                       // Are we dealing with an attachment?
                       mTest := ARequestInfo.ContentType;
                       if AnsiSameText(Fetch(mTest, ';'), 'multipart/form-data') then  //
                       begin
                            // Save all attachments...

                            // Accept file here
                            mSaveDir := IncludeTrailingPathDelimiter(Glob_Reg_VMSDir)+
                                        IncludeTrailingPathDelimiter(apOfficeUpload);

                            if not DirectoryExists(mSaveDir) then
                            begin
                                 ForceDirectories(mSaveDir);
                            end;


                            // RECEIVE THE FILE
                            mUploadFile := RecordParseCustomDelimiter(mCommandResult, '?', 1);
                            try // PostStream.Size might be invalid

                               mContentType := ARequestInfo.ContentType;
                               if AnsiSameText(Fetch(mContentType, ';'), 'multipart/form-data') then   // 'multipart/form-data'
                               begin
                                    //ARequestInfo.PostStream.Position := 0;
                                    DecodeFormData(mSaveDir, mContentType, ARequestInfo.PostStream);
                               end;
                            except
                            end;


                       end
                       else
                       begin
                            // Request data
                            //mStream := ARequestInfo.PostStream;
                            mSize := ARequestInfo.PostStream.Size;
                            if mSize > 0 then
                            begin
                                 mStream.LoadFromStream(ARequestInfo.PostStream);//  (mParams, mStream.Size);

                                 mStreamString := '';
                                 // Set the size of the buffer to read into
                                 SetLength( mStreamString, mSize );
                                 mStream.Read(mStreamString[1], mSize);
                                 //ARequestInfo.PostStream.SaveToFile('E:\enGenic\Web\upload\test.jpg');
                            end;
                       end;

                    finally
                       mStream.Free;
                    end;
               end; // not mTeleFlowMode
          end
          else
          begin

               // ========================================
               // - Check for standard sub-directories
               // Does it exist?  if not, try a few other approaches
               if not FileExists(LocalDoc) then
               begin
                    mDocExt := UpperCase(ExtractFileExt(LocalDoc));


                    /////////////////////////////////////////////
                    /// REMEMBER:  These are VoiceOffice Files...
                    ///            they will be served on port: Glob_Reg_HTTPVMSP
                    /////////////////////////////////////////////


                    // Voice Messages
                    if (mDocExt = '.WAV') then
                    begin
                         LocalDoc := mDocPath + IncludeTrailingPathDelimiter('vmail') + mDocFileName;
                    end;

                    // System logs files
                    if (mDocExt = '.LOG') then
                    begin
                         // From the TeleFlow install
                         try
                            // May not be on a teleflow system
                            ReadTeleFlowRegistry;
                         except
                         end;
                         LocalDoc := IncludeTrailingPathDelimiter(GlobRegLOGDIRECTORY) + mDocFileName;

                         LogEventMessage('LogFile Request: '+LocalDoc, True, mPeerIP );
                    end;

                    // Stats and reports
                    if (mDocExt = '.REP') or (mDocExt = '.TXT') then
                    begin
                         LocalDoc := mDocPath + IncludeTrailingPathDelimiter('stats') + mDocFileName;
                    end;

                    // Executables and other information
                    if (mDocExt = '.EXE') or (mDocExt = '.ZIP')  then
                    begin
                         LocalDoc := mDocPath + IncludeTrailingPathDelimiter('updates') + mDocFileName;
                    end;
               end;


               // Did we find a Download (A file to save, not for example a Web page to serve)
               if FileExists(LocalDoc) then
               begin
                    // If the Download path is the save as the Web Page, then web override.
                    if AnsiSameText(IncludeTrailingPathDelimiter(Glob_Reg_HTTPRoot),
                                    IncludeTrailingPathDelimiter(mDocPath)) then
                    begin
                         mVMSRequest := False;
                         LogEventMessage('Web File Located: '+LocalDoc, True, mPeerIP);
                    end
                    else
                    begin
                         mVMSRequest := True;
                         LogEventMessage('Download File Located: '+LocalDoc, True, mPeerIP);
                    end;
               end
               else
               begin
                    // Interprete the command to it's final path (avoid sending files in parent folders)
                    LocalDoc    := ExpandFilename(Glob_Reg_HTTPRoot + mDocFileName);
                    mVMSRequest := False;

                    (*
                    // I *HATE* doing things this way, BUT.....
                    // Quick test to see if we are wanting a systemhealth file
                    mDocExt := UpperCase(ExtractFileExt(LocalDoc));
                    // Executables and other information
                    if (mDocExt = '.EXE') or (mDocExt = '.ZIP')  then
                    begin
                         mDocPath := AcquireFileBaseDirectory();
                         // Switch to another location
                         LocalDoc := mDocPath + IncludeTrailingPathDelimiter('updates') + ARequestInfo.Document;

                         mVMSRequest := True;
                         LogEventMessage('SystemHealth File: '+LocalDoc, True, mPeerIP);
                         // WHY: Because you cant link to an html file on a different port for a download.  Trust me to find that one.
                    end;
                    *)
               end;

               LogEventMessage('HTTP'+iff(Glob_Reg_HTTPStealth = 1,'(STEALTH)','')+': '+LocalDoc, True, mPeerIP);

               ///////////////////////////////////////////////
               // Test for enGenic Created pages
               ///////////////////////////////////////////////

               // Are we working with params, or something else
               // Note: UnparsedParams come after the ?
               //   eg: http://localhost:5048/conferenceaccess.html?ConfNum=123123&ConfCon=123123123
               //    =: ConfNum=123123&ConfCon=Vern+Baker
               if Copy(ARequestInfo.UnparsedParams,1,1) <> '' then
               begin
                    // The client sent a command with parameters
                    mTCPResult := ''; //TCPMessageInteractManager(ARequestInfo.UnparsedParams);
                    // Unformatted response
                    AResponseInfo.ContentText :=  mTCPResult;

                    // Generate Conferencing HTML & XML
                    if ARequestInfo.Document = '/'+cConferenceAccessHtml then
                    begin
                         mParams   := ARequestInfo.UnparsedParams;
                         // Conference Id
                         mField01  := RecordParseCustomDelimiter(RecordParseCustomDelimiter(mParams,'&',1),'=',1);
                         // Contact Add
                         mField02  := RecordParseCustomDelimiter(RecordParseCustomDelimiter(mParams,'&',2),'=',2);
                         // Remove + and return to spaces
                         mField02  := StringReplace(mField02,'+',' ', [rfReplaceAll, rfIgnoreCase]);

                         // Generate the WebPage
                         // - p1: Requestor host name
                         // - p2: Conference number
                         // - p3: Conference ???
                         mHTMLData := NetHubGenerateConferenceAccess(mHostByRequest, mField01, mField02);
                    end;
               end;

               // SystemHealth
               /////////////////////////
               if pos('systemhealth.html',lowercase(ARequestInfo.Document)) > 0 then
               begin
                    // NOTE: This function moved to TFSentinel.. but, check that That it is running
                    (*
                    // build it, write it, serve it
                    //LogEventMessage('Building: '+LocalDoc, True, mPeerIP);
                    try
                       // May not be on a teleflow system
                       ReadTeleFlowRegistry;
                    except
                    end;

                    // If not screen mode, then lock
                    {$ifndef SCREENMODE}
                    ConsoleLock.Acquire;
                    {$endif}
                    try
                       // Get system health, and scan for logs files
                       mHTMLData := NetHubGenerateSystemHealth(mHostByRequest, LocalDoc, True);
                    finally
                       {$ifndef SCREENMODE}
                       ConsoleLock.Release;
                       {$endif}
                    end;
                    *)

                    // Check to see if Sentinel is Installed (ConstStandardInclude)
                    if (AnotherServiceGetStatus('',cSentinelService) = -1) then
                    begin
                         mHTMLData.Add( HTMLWrapErrorMessage('ERROR: TFSentinel is not Installed'));
                    end
                    else
                    // Check to see if Sentinel is running
                    if not (AnotherServiceRunning ('',cSentinelService)) then
                    begin
                         mHTMLData.Add( HTMLWrapErrorMessage('ERROR: TFSentinel is not Started'));
                    end;

                    mSpecialMode := True;
               end;

               // SystemHealth RSS feed // 'systemhealth_rss.xml'
               /////////////////////////
               if pos(cSystemHealthRSS,lowercase(ARequestInfo.Document)) > 0 then
               begin
                    // If not screen mode, then lock
                    {$ifndef SCREENMODE}
                    ConsoleLock.Acquire;
                    {$endif}
                    try
                       // Get system health reports, and aggregate them
                       mHTMLData := CreateSystemHealthRSSXML;
                    finally
                       {$ifndef SCREENMODE}
                       ConsoleLock.Release;
                       {$endif}
                    end;

                    mSpecialMode := True;
               end;

               if pos('monitor.html',lowercase(ARequestInfo.Document)) > 0 then
               begin
                    // If not screen mode, then lock
                    {$ifndef SCREENMODE}
                    ConsoleLock.Acquire;
                    {$endif}
                    try
                       // Get system health, and scan for logs files
                       mHTMLData := NetHubGenerateMonitor(mHostByRequest, LocalDoc);
                    finally
                       {$ifndef SCREENMODE}
                       ConsoleLock.Release;
                       {$endif}
                    end;

                    mSpecialMode := True;
               end;

               // Check for some images
               if pos('systemhealth.jpg',lowercase(ARequestInfo.Document)) > 0 then
               begin
                    // Let the header get served for special pages
                    mSpecialMode := True;
               end;
               
               if pos('monitor.jpg',lowercase(ARequestInfo.Document)) > 0 then
               begin
                    // Let the header get served for special pages
                    mSpecialMode := True;
               end;


               // SpecialAccess
               /////////////////////////
               if pos('adminaccess.html',lowercase(ARequestInfo.Document)) > 0 then
               begin
                    // Give the age of the files and contents as well.

                    // build it, write it, serve it
                    //LogEventMessage('Building: '+LocalDoc, True, mPeerIP);
                    try
                       // May not be on a teleflow system
                       ReadTeleFlowRegistry;
                    except
                    end;

                   // mHTMLData := NetHubGenerateAdminAccess(mHostByRequest, LocalDoc);

                    mSpecialMode := True;
               end;
               // Check for some images
               if pos('adminaccess.jpg',lowercase(ARequestInfo.Document)) > 0 then
               begin
                    // Let the header get served for special pages
                    mSpecialMode := True;
               end;
          end;



          // Only serve if not in Stealth or special pages are built
          if (Glob_Reg_HTTPStealth = 0) or mSpecialMode then
          begin

               if not mSpecialMode then
               begin
                    // Test for a command
                    // - This will start with '/&', so ignor that bit
                    // 0 = HTTP mode
                    // Note:  I added ARequestInfo.UnparsedParams in so that params could be passed.  Dec 2006
                    mParameters   := '';

                    if ARequestInfo.Document <> '' then
                    begin
                         // Take the entire parameters list
                         mParameters := ARequestInfo.Document // // Get rid of '/&' Copy(ARequestInfo.Document,3,9999);
                    end;

                    // See if we are building on to the end
                    if ARequestInfo.UnparsedParams <> '' then
                    begin
                         // Do not pass posted files as parameters
                         mCommandCheck :=  RecordParseCustomDelimiter(mParameters,'~', cTFCommCommand);

                         // Don't pass information if it is a file
                         if (not AnsiSameText(mCommandCheck, cTFSendFile)) and
                            (not AnsiSameText(mCommandCheck, cTFSendScreenCap)) then
                         begin
                              // Build the parameters onto the end
                              mParameters := mParameters + '?' + ARequestInfo.UnparsedParams;
                         end;
                    end;


                    // Are these submittable commands?
                    mProcessIt := False;
                    if mTeleFlowMode then
                    begin
                         // TeleFlow Web Page managment
                         ///////////////////////////////

                         // mField
                         mProcessIt := True;
                    end
                    else
                    begin
                         // Standard Web Page managment
                         ///////////////////////////////

                         // Things like XML Posts etc
                         if mStreamString <> '' then
                         begin
                              mParameters := mParameters + '?' + mStreamString;
                         end;


                         // Decide if we should make it into one
                         // - First 200 chars should be enough to cover the page name
                         if (Pos('<?xml', lowercase(Copy(mParameters,1,200))) > 0) then
                         begin
                              mProcessIt := True;

                              // Log incoming XML posts
                              LogEventMessage('XML Post: '+mParameters, True, mPeerIP);
                         end;
                    end;


                    /////////////////////////
                    // Process any commands sent to NetHub
                    /////////////////////////
                    if mProcessIt then
                    begin
                         mCommandResult := ProcessCommands(0, mCommandIP, mParameters, GlobVersionNetHub, mCode);
                         if mCommandResult = cTFPutAccepted then
                         begin
                              LogEventMessage(GlobCommandLogEntry, True, mPeerIP);
                         end;
                    end;
               end;

               /////////////////////////
               // SCREEN ONLY
               // - Refresh data
               {$ifdef SCREENMODE}
               CommandsMemoUpdate;
               {$endif}
               /////////////////////////


               if mCommandResult <> '' then
               begin
                    // Check EMAIL errors
                    if mCode = cTFCommErrEMAIL then
                    begin
                         // Log the actual error to the
                         LogEventMessage('SendEmailError: '+mCommandResult, True, mPeerIP);
                         // Set the command to just error
                         mCommandResult := 'ERROR';
                    end;


                    // SEND File to NetHub (NetHub receives it)
                    if mCode = cTFCommReceive then
                    begin
                         // Accept file here
                         mSaveDir := IncludeTrailingPathDelimiter(Glob_Reg_VMSDir)+
                                     IncludeTrailingPathDelimiter(apOfficeUpload);

                         if not DirectoryExists(mSaveDir) then
                         begin
                              ForceDirectories(mSaveDir);
                         end;

                         // RECEIVE THE FILE
                         mUploadFile := RecordParseCustomDelimiter(mCommandResult, '?', 1);
                         try // PostStream.Size might be invalid


                            // See if we are receving a MIME file (from a Web site upload)
                            mContentType := ARequestInfo.ContentType;
                            if AnsiSameText(Fetch(mContentType, ';'), 'multipart/form-data') then
                            begin
                                 ARequestInfo.PostStream.Position := 0;
                                 DecodeFormData(mSaveDir, mContentType, ARequestInfo.PostStream);
                            end
                            else
                            begin
                                 // Save the programmatic transfer
                                 TMemoryStream(ARequestInfo.PostStream).SaveToFile(IncludeTrailingPathDelimiter(mSaveDir) + mUploadFile);
                            end;

                            Glob_Bytes_Received := Glob_Bytes_Received + ARequestInfo.PostStream.Size;
                         except
                         end;

                    end;

                    // SEND a Screencapture to NetHub (NetHub receives it)
                    if mCode = cTFCommScreenCap then
                    begin
                         // Accept file here
                         mSaveDir := IncludeTrailingPathDelimiter(Glob_Reg_VMSDir)+
                                     IncludeTrailingPathDelimiter(apOfficeUpload)+
                                     IncludeTrailingPathDelimiter('ScreenCaptures');

                         if not DirectoryExists(mSaveDir) then
                         begin
                              ForceDirectories(mSaveDir);
                         end;
                         TMemoryStream(ARequestInfo.PostStream).SaveToFile(IncludeTrailingPathDelimiter(mSaveDir) + mCommandResult);
                         try // PostStream.Size might be invalid
                            Glob_Bytes_Received := Glob_Bytes_Received + ARequestInfo.PostStream.Size;
                         except
                         end;
                    end;

                    // LOG the result
                    if (RecordParseCustomDelimiter(mCommandResult, '~', cTFCommCommand) = cTFLog) then
                    begin
                         // Drop in what we have - into a file that creates monthly
                         mFile := IncludeTrailingPathDelimiter(Glob_Reg_LogDir)+LowerCase(cAppName+'-'+cTFLog+'-'+FormatDateTime('yyyy"-"mm',Now)+'.log');
                         CreateLogEntry(mFile, mCommandResult)
                    end
                    else
                    begin
                         // Just respond with text from ProcessCommads
                         AResponseInfo.ResponseNo  := 200; // Okay
                         AResponseInfo.ContentText := mCommandResult;
                         LogEventMessage('CommandReponse: '+mCommandResult, True, mPeerIP);


                         if AnsiSameText(ARequestInfo.Command, 'POST') then
                         begin
                              // AResponseInfo.ServeFile(AContext,'Hello.txt');
                         end
                         else
                         begin

                         end;

                    end;
               end
               else
               begin
                    // Dont process under certain circumstances
                    if not mSpecialMode then
                    begin
                         // Default document (index.html) for folder
                         if (not FileExists(LocalDoc)) or (ARequestInfo.Document = '/index.html') then
                         // and DirectoryExists(LocalDoc) and
                         //   FileExists(ExpandFileName(LocalDoc + '/index.html')) then
                         begin
                              // build it, write it, serve it
                              //LocalDoc := ExpandFileName(LocalDoc + '/index.html');
                              //LogEventMessage('Building: '+LocalDoc, True, mPeerIP);
                              try
                                 LocalDoc :=  IncludeTrailingPathDelimiter(Glob_Reg_HTTPRoot)+ 'index.html';
                                 mHTMLData := NetHubGenerateHomePage(mHostByRequest, LocalDoc);

                                 //DeleteFile(LocalDoc);
                                 //mHTMLData.SaveToFile(LocalDoc);

                                 // Return error message for other pages
                                 if (not FileExists(LocalDoc)) and
                                    (not AnsiSameText(ARequestInfo.Document,'/index.html'))  then
                                 begin
                                      AResponseInfo.ResponseNo  := 404; // Not found
                                 end;


                              except
                              end;
                         end;

                         // Conference Enter page
                         if (ARequestInfo.Document = '/'+cConferenceHtml) then
                         begin
                              // build it, write it, serve it
                              //LocalDoc := ExpandFileName(LocalDoc + '/index.html');
                              //LogEventMessage('Building: '+LocalDoc, True, mPeerIP);
                              try
                                 LocalDoc  := IncludeTrailingPathDelimiter(Glob_Reg_HTTPRoot)+ cConferenceHtml;
                                 mHTMLData := NetHubGenerateConference(mHostByRequest, LocalDoc);

                                 //DeleteFile(LocalDoc);
                                 //mHTMLData.SaveToFile(LocalDoc);
                              except
                              end;
                         end;

                         // WordVerify Enter page
                         if (ARequestInfo.Document = '/'+cWordVerifyHtml) then
                         begin
                              // build it, write it, serve it
                              //LocalDoc := ExpandFileName(LocalDoc + '/index.html');
                              //LogEventMessage('Building: '+LocalDoc, True, mPeerIP);
                              try
                                 LocalDoc :=  IncludeTrailingPathDelimiter(Glob_Reg_HTTPRoot)+ cWordVerifyHtml;
                                 mHTMLData := NetHubGenerateWordVerify(mHostByRequest, LocalDoc);

                                 //DeleteFile(LocalDoc);
                                 //mHTMLData.SaveToFile(LocalDoc);
                              except
                              end;
                         end;
                    end; // mSpecialMode


                    // Did we create the page / file in memory
                    if mHTMLData.Count > 0 then
                    begin
                         ////////////////////////////////////
                         // Serve back the memory stream
                         ////////////////////////////////////
                         mDataStream := TMemoryStream.Create;
                         try
                            mHTMLData.SaveToStream(mDataStream);
                            AResponseInfo.FreeContentStream := false;
                            AResponseInfo.ContentStream := mDataStream;
                            try
                               AResponseInfo.WriteHeader;
                               AResponseInfo.WriteContent;
                               Glob_Bytes_Served := Glob_Bytes_Served + AResponseInfo.ContentLength;
                            finally
                               AResponseInfo.ContentStream := nil;
                            end;
                         finally
                            mDataStream.Free;
                         end;
                    end
                    else if FileExists(LocalDoc) then // File exists
                    begin
                         // VMS Mode is generally a file to download and save
                         // - Should be a in a different directory than the HTML files
                         if mVMSRequest then
                         begin
                              // Normal document request
                              // Send the document back
                              try
                                 // Let the browser know we have a file attachment to return
                                 AResponseInfo.CustomHeaders.Values['Content-Disposition'] :=
                                              'attachment; filename="'+ExtractFileName( StringReplace(LocalDoc, '/','', [rfReplaceAll]))+'"';
                                 LogEventMessage(AResponseInfo.CustomHeaders.Values['Content-Disposition'], True, mPeerIP);


                                 ////////////////////////////////////
                                 // Serve out a disk file
                                 ////////////////////////////////////
                                 //ByteSent := HTTPServer.ServeFile(AThread, AResponseInfo, LocalDoc);
                                 ByteSent := AResponseInfo.ServeFile(AContext, LocalDoc);

                                 Glob_Bytes_Served := Glob_Bytes_Served + ByteSent;

                                 (* Another way...
                                 mDataStream := TMemoryStream.Create;
                                 try
                                    mDataStream.LoadFromFile(LocalDoc);
                                    AResponseInfo.FreeContentStream := false;
                                    AResponseInfo.ContentStream := mDataStream;
                                    try
                                       AResponseInfo.WriteHeader;
                                       AResponseInfo.WriteContent;
                                       Glob_Bytes_Served := Glob_Bytes_Served + AResponseInfo.ContentLength;
                                    finally
                                       AResponseInfo.ContentStream := nil;
                                    end;
                                 finally
                                    mDataStream.Free;
                                 end;
                                 *)


                                 LogEventMessage(Format('SERVING file %s (%d bytes / %d bytes sent) to %s:%d',
                                                       [LocalDoc, ByteSent, FileSizeByName(LocalDoc),
                                                        TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerIP,
                                                        TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerPort]),
                                                 True, mPeerIP);
                                 ////////////////////////
                                 // A file was serviced
                                 inc(GlobTotalServed);
                                 ////////////////////////
                              except
                                 // We had a major failure
                                 on E: Exception do
                                 begin
                                      HandleErrorMessage(E);
                                 end;
                              end;
                         end
                         else
                         begin
                              if (AnsiSameText(Copy(LocalDoc, 1, Length(Glob_Reg_HTTPRoot)), Glob_Reg_HTTPRoot)) then
                              begin
                                   // File down in dir structure
                                   if AnsiSameText(ARequestInfo.Command, 'HEAD') then
                                   begin
                                        // HEAD request, don't send the document but still send back it's size
                                        ResultFile := TFileStream.create(LocalDoc, fmOpenRead or
                                          fmShareDenyWrite);
                                        try
                                          AResponseInfo.ResponseNo    := 200;
                                          AResponseInfo.ContentType   := GetMIMEType(LocalDoc);
                                          AResponseInfo.ContentLength := ResultFile.Size;
                                        finally
                                          ResultFile.Free;
                                            // We must free this file since it won't be done by the web server component
                                        end;
                                   end
                                   else
                                   begin
                                        try
                                           // Normal document request
                                           // Send the document back

                                           ////////////////////////////////////
                                           // Serve out a disk file
                                           ////////////////////////////////////
                                           ByteSent := AResponseInfo.ServeFile(AContext, LocalDoc);
                                           Glob_Bytes_Served := Glob_Bytes_Served + ByteSent;
                                           
                                           (* Another way...
                                           mDataStream := TMemoryStream.Create;
                                           try
                                              mDataStream.LoadFromFile(LocalDoc);
                                              AResponseInfo.FreeContentStream := false;
                                              AResponseInfo.ContentStream := mDataStream;
                                              try
                                                 AResponseInfo.WriteHeader;
                                                 AResponseInfo.WriteContent;
                                                 Glob_Bytes_Served := Glob_Bytes_Served + AResponseInfo.ContentLength;
                                              finally
                                                 AResponseInfo.ContentStream := nil;
                                              end;
                                           finally
                                              mDataStream.Free;
                                           end;
                                           *)

                                           LogEventMessage(Format('SERVING file %s (%d bytes / %d bytes sent) to %s:%d',
                                                                 [LocalDoc, ByteSent, FileSizeByName(LocalDoc),
                                                                  TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerIP,
                                                                  TIdIOHandlerSocket(AContext.Connection.IOHandler).Binding.PeerPort]),
                                                           True, mPeerIP);
                                           ////////////////////////
                                           // A file was serviced
                                           inc(GlobTotalServed);
                                           ////////////////////////

                                           // Double Check
                                           if UpperCaseCompare('I/O error',LocalDoc) then
                                           begin
                                                mFile := IncludeTrailingPathDelimiter(Glob_Reg_LogDir)+LowerCase(cAppName+'-'+FormatDateTime('yyyy"-"mm"-"dd"',Now)+'.log');
                                                CreateLogEntry(mFile,'ERROR: 103')
                                           end;

                                        except
                                           // We had a major failure
                                           on E: Exception do
                                           begin
                                                HandleErrorMessage(E);
                                           end;
                                        end;
                                   end;
                              end
                              else
                              begin
                                   AccessDenied;
                              end;
                         end;
                    end
                    else
                    begin
                         AResponseInfo.ResponseNo  := 404; // Not found
                         AResponseInfo.ContentText := '<html><head><title>Error</title></head><body><h1>' +
                                                      AResponseInfo.ResponseText + '</h1></body></html>';
                         LogEventMessage('*** ERROR '+IntToStr(AResponseInfo.ResponseNo)+': '+LocalDoc, True, mPeerIP);
                    end;
               end; // mCommandResult
          end;  // Serve only under certain conditions
     end;
  finally
     mHTMLData.Free;
  end;
//end; // HTTPServerCommandGet  (Leave off... must be in calling code)

