unit ServiceWorkUnit;

interface

uses Messages,SysUtils,Classes,Windows,WinSvc;


   function AnotherServiceGetStatus(sMachine,sService : string ): Integer; // DWord;
   function AnotherServiceRunning(sMachine,sService : string ): boolean;
   function AnotherServiceStart(sMachine,sService : string ): boolean;
   function AnotherServiceStop(sMachine,sService : string ): boolean;



implementation
//-------------------------------------
(*
Block #2 : Delphi code. Right click chkss2.pas to download.

Both "ServiceRunning()" and "ServiceStopped()" functions
expect a standard machine name and a service name. The machine
name can be empty if you're checking a service running on the local
machine. For example:
*)

(*
if( ServiceRunning(
  '\\ComputerName',
  'alerter' ) )then
begin
  // "alerter" service on \\ComputerName
  // is running
  // take appropriate action here
end;


if( ServiceRunning(
  '',
  'alerter' ) )then
begin
  // "alerter" service on the
  // local computer is running
end;


if( ServiceStopped(
  '',
  'alerter' ) )then
begin
  // "alerter" service on the
  // local computer is in the
  // "stopped" state (not running)
end
*)

//-------------------------------------
// get service status
//
// return status code if successful
// -1 if not
//
// return codes:
//   SERVICE_STOPPED
//   SERVICE_RUNNING
//   SERVICE_PAUSED
//
// following return codes
// are used to indicate that
// the service is in the
// middle of getting to one
// of the above states:
//   SERVICE_START_PENDING
//   SERVICE_STOP_PENDING
//   SERVICE_CONTINUE_PENDING
//   SERVICE_PAUSE_PENDING
//
// sMachine:
//   machine name, ie: \\SERVER   empty = local machine
//
// sService
//   service name, ie: Alerter
//
function AnotherServiceGetStatus(
  sMachine,
  sService : string ) : Integer;
var 
  //
  // service control
  // manager handle
  schm,
  //
  // service handle 
  schs   : SC_Handle; 
  //
  // service status
  ss     : TServiceStatus;
  // 
  // current service status
  dwStat : Integer;
begin
  dwStat := -1;


  // connect to the service
  // control manager
  schm := OpenSCManager(
    PChar(sMachine),
    Nil,
    SC_MANAGER_CONNECT);


  // if successful...
  if(schm > 0)then
  begin
    // open a handle to
    // the specified service
    schs := OpenService(
      schm,
      PChar(sService),
      // we want to
      // query service status
      SERVICE_QUERY_STATUS);


    // if successful...
    if(schs > 0)then
    begin
      // retrieve the current status
      // of the specified service
      if(QueryServiceStatus(
           schs,
           ss))then
      begin
        dwStat := ss.dwCurrentState;
      end;


      // close service handle
      CloseServiceHandle(schs);
    end;


    // close service control
    // manager handle
    CloseServiceHandle(schm);
  end;


  Result := dwStat;
end;


(*
Block #1 : Delphi code. Right click chkss.pas to download.

Now that we have a basic function -- "ServiceGetStatus()"
 -- to get the status of a specified service running on a specified
machine,
let's simplify the process even more with the following
"ServiceRunning()" and "ServiceStopped()" functions.
*)


//-------------------------------------
// return TRUE if the specified
// service is running, defined by
// the status code SERVICE_RUNNING.
// return FALSE if the service
// is in any other state, including 
// any pending states 
//
function AnotherServiceRunning(
  sMachine,
  sService : string ) : boolean;
begin 
  Result := SERVICE_RUNNING = 
    AnotherServiceGetStatus(
      sMachine, sService ); 
end; 


//------------------------------------- 
// return TRUE if the specified
// service was stopped, defined by 
// the status code SERVICE_STOPPED.
//
function AnotherServiceStopped(
  sMachine, 
  sService : string ) : boolean;
begin
  Result := SERVICE_STOPPED =
    AnotherServiceGetStatus(
      sMachine, sService ); 
end;


//
// start service
//
// return TRUE if successful
//
// sMachine:
//   machine name, ie: \SERVER
//   empty = local machine
//
// sService
//   service name, ie: Alerter
//
function AnotherServiceStart(
  sMachine,
  sService : string ) : boolean;
var
  //
  // service control
  // manager handle
  schm,
  //
  // service handle
  schs   : SC_Handle;
  //
  // service status
  ss     : TServiceStatus;
  //
  // temp char pointer
  psTemp : PChar;
  //
  // check point
  dwChkP : Longword;
begin
  //ss.dwCurrentState := -1;
  
  // connect to the service
  // control manager
  schm := OpenSCManager(
    PChar(sMachine),
    Nil,
    SC_MANAGER_CONNECT);

  // if successful...
  if(schm > 0)then
  begin
    // open a handle to
    // the specified service
    schs := OpenService(
      schm,
      PChar(sService),
      // we want to
      // start the service and
      SERVICE_START or
      // query service status
      SERVICE_QUERY_STATUS);

    // if successful...
    if(schs > 0)then
    begin
      psTemp := Nil;
      if(StartService(
           schs,
           0,
           psTemp))then
      begin
        // check status
        if(QueryServiceStatus(
             schs,
             ss))then
        begin
          while(SERVICE_RUNNING
            <> ss.dwCurrentState)do
          begin
            //
            // dwCheckPoint contains a
            // value that the service
            // increments periodically
            // to report its progress
            // during a lengthy
            // operation.
            //
            // save current value
            //
            dwChkP := ss.dwCheckPoint;

            //
            // wait a bit before
            // checking status again
            //
            // dwWaitHint is the
            // estimated amount of time
            // the calling program
            // should wait before calling
            // QueryServiceStatus() again
            //
            // idle events should be
            // handled here...
            //
            Sleep(ss.dwWaitHint);

            if(not QueryServiceStatus(
                 schs,
                 ss))then
            begin
              // couldn't check status
              // break from the loop
              break;
            end;

            if(ss.dwCheckPoint <
              dwChkP)then
            begin
              // QueryServiceStatus
              // didn't increment
              // dwCheckPoint as it
              // should have.
              // avoid an infinite
              // loop by breaking
              break;
            end;
          end;
        end;
      end;

      // close service handle
      CloseServiceHandle(schs);
    end;

    // close service control
    // manager handle
    CloseServiceHandle(schm);
  end;

  // return TRUE if
  // the service status is running
  Result :=
    SERVICE_RUNNING =
      ss.dwCurrentState;
end;



//
// stop service
//
// return TRUE if successful
//
// sMachine:
//   machine name, ie: \SERVER
//   empty = local machine
//
// sService
//   service name, ie: Alerter
//
function AnotherServiceStop(
  sMachine,
  sService : string ) : boolean;
var
  //
  // service control
  // manager handle
  schm,
  //
  // service handle
  schs   : SC_Handle;
  //
  // service status
  ss     : TServiceStatus;
  //
  // check point
  dwChkP : DWord;
begin
  // connect to the service
  // control manager
  schm := OpenSCManager(
    PChar(sMachine),
    Nil,
    SC_MANAGER_CONNECT);

  // if successful...
  if(schm > 0)then
  begin
    // open a handle to
    // the specified service
    schs := OpenService(
      schm,
      PChar(sService),
      // we want to
      // stop the service and
      SERVICE_STOP or
      // query service status
      SERVICE_QUERY_STATUS);

    // if successful...
    if(schs > 0)then
    begin
      if(ControlService(
           schs,
           SERVICE_CONTROL_STOP,
           ss))then
      begin
        // check status
        if(QueryServiceStatus(
             schs,
             ss))then
        begin
          while(SERVICE_STOPPED
            <> ss.dwCurrentState)do
          begin
            //
            // dwCheckPoint contains a
            // value that the service
            // increments periodically
            // to report its progress
            // during a lengthy
            // operation.
            //
            // save current value
            //
            dwChkP := ss.dwCheckPoint;

            //
            // wait a bit before
            // checking status again
            //
            // dwWaitHint is the
            // estimated amount of time
            // the calling program
            // should wait before calling
            // QueryServiceStatus() again
            //
            // idle events should be
            // handled here...
            //
            Sleep(ss.dwWaitHint);

            if(not QueryServiceStatus(
                 schs,
                 ss))then
            begin
              // couldn't check status
              // break from the loop
              break;
            end;

            if(ss.dwCheckPoint <
              dwChkP)then
            begin
              // QueryServiceStatus
              // didn't increment
              // dwCheckPoint as it
              // should have.
              // avoid an infinite
              // loop by breaking
              break;
            end;
          end;
        end;
      end;

      // close service handle
      CloseServiceHandle(schs);
    end;

    // close service control
    // manager handle
    CloseServiceHandle(schm);
  end;

  // return TRUE if
  // the service status is stopped
  Result :=
    SERVICE_STOPPED =
      ss.dwCurrentState;
end;

end.
