unit TCPIPWorkUnit;

interface

uses classes, IdHTTP, CalcWorkUnit;

function DNSLookupName(const Name: string): String;
function GetLocalIPs: TStringList;
function GetLocalIP: string;
function HTTPLog(pHost: String; pPort: Integer; pLogCode, pLogMessage: String): Boolean;
function HTTPGetFile(pHost: String; pPort: Integer; pFileName, pFileTo, pFileError: String): Boolean;
function HTTPGetTextFile(pWebAddress: String): TStringList;
function HTTPSendXML(pHost: String; pPort: Integer; pPage: String; pFileStrings: TStringList): String;
function HTTPSendFile(pHost: String; pPort: Integer; pCommand: String; pFileName: String): String;
function HTTPSubmitURLRequest(pURLRequest: String): String;
function HTTPSendCommandToNetHub(pTimeOutms: Integer; pHost: String; pPort: Integer; pCommand: String): String;
function HTTPGetProgramUpdate(pHost: String; pPort: Integer; pThisProgramExe, pVersion, pUserName, pComputerName: String): String;
function HTTPLastErrorMessage: String;
function PrefixHTTP(pHost: String): String;
function HTTPAddressBreakoutIP(pAddress: String): String;
function HTTPAddressBreakoutPort(pAddress: String): Integer;

(* Error list

Socket error 10004 - Interrupted function call
Socket error 10013 - Permission denied
Socket error 10014 - Bad address
Socket error 10022 - Invalid argument
Socket error 10024 - Too many open files
Socket error 10035 - Resource temporarily unavailable
Socket error 10036 - Operation now in progress
Socket error 10037 - Operation already in progress
Socket error 10038 - Socket operation on non-socket
Socket error 10039 - Destination address required
Socket error 10040 - Message too long
Socket error 10041 - Protocol wrong type for socket
Socket error 10042 - Bad protocol option
Socket error 10043 - Protocol not supported
Socket error 10044 - Socket type not supported
Socket error 10045 - Operation not supported
Socket error 10046 - Protocol family not supported
Socket error 10047 - Address family not supported by protocol family
Socket error 10048 - Address already in use
Socket error 10049 - Cannot assign requested address
Socket error 10050 - Network is down
Socket error 10051 - Network is unreachable
Socket error 10052 - Network dropped connection on reset
Socket error 10053 - Software caused connection abort
Socket error 10054 - Connection reset by peer
Socket error 10055 - No buffer space available
Socket error 10056 - Socket is already connected
Socket error 10057 - Socket is not connected
Socket error 10058 - Cannot send after socket shutdown
Socket error 10060 - Connection timed out
Socket error 10061 - Connection refused
Socket error 10064 - Host is down
Socket error 10065 - No route to host
Socket error 10067 - Too many processes
Socket error 10091 - Network subsystem is unavailable
Socket error 10092 - WINSOCK.DLL version out of range
Socket error 10093 - Successful WSAStartup not yet performed
Socket error 10094 - Graceful shutdown in progress
Socket error 11001 - Host not found
Socket error 11002 - Non-authoritative host not found
Socket error 11003 - This is a non-recoverable error
Socket error 11004 - Valid name, no data record of requested type
*)


implementation

uses
   winsock, sysutils;

const
{$I ConstStandardInclude.pas}

var
   GlobMessage: String;

// Return just the Address
// - http://www.engenic.com:934
// - http://www.engenic.com
// - https://23.241.12.43/something.html
function HTTPAddressBreakoutIP(pAddress: String): String;
var
   mPos   : Integer;
   mAddr  : String;
begin
     mAddr := pAddress;

     // Get rid of the http://
     mPos  := Pos('//', mAddr);
     if mPos > 0 then
     begin
          mAddr := Copy(mAddr, mPos+2, 999);
     end;

     // See if we have anything at the end... and remove it
     mPos  := Pos('/', mAddr);
     if mPos > 0 then
     begin
          mAddr := Copy(mAddr, 0, mPos - 1);
     end;

     // See if we have a Port
     mPos  := Pos(':', mAddr);
     if mPos > 0 then
     begin
          mAddr := Copy(mAddr, 0, mPos - 1);
     end;

     result := mAddr;
end; // HTTPAddressBreakoutIP


function HTTPAddressBreakoutPort(pAddress: String): Integer;
var
   mPos   : Integer;
   mAddr  : String;
begin
     mAddr := pAddress;

     // Get rid of the http://
     mPos  := Pos('//', mAddr);
     if mPos > 0 then
     begin
          mAddr := Copy(mAddr, mPos+2, 999);
     end;

     // See if we have anything at the end... and remove it
     mPos  := Pos('/', mAddr);
     if mPos > 0 then
     begin
          mAddr := Copy(mAddr, 0, mPos - 1);
     end;

     // See if we have a Port
     mPos  := Pos(':', mAddr);
     if mPos > 0 then
     begin
          result := StrToIntDef(Copy(mAddr, mPos + 1, 999),80);
     end
     else
     begin
          result := StrToIntDef(mAddr, 80);
     end;
end; // HTTPAddressBreakoutPort


// Get a list of all the Local IPs
// eg: Memo1.Lines.AddStrings(GetLocalIPs);
function GetLocalIPs: TStringList;
type
  PPInAddr= ^PInAddr;
var
  wsaData    : TWSAData;
  HostInfo   : PHostEnt;
  HostName   : Array[0..255] of Char;
  Addr       : PPInAddr;
  TcpipList  : TStringlist;
begin
     TcpipList := TStringlist.Create;
     result    :=tcpiplist;

     try
        if WSAStartup($0102, wsaData)<>0 then exit;
        try
          if gethostname(HostName, SizeOf(HostName)) <> 0 then exit;
          HostInfo:= gethostbyname(HostName);
          if HostInfo=nil then Exit;
          Addr:=Pointer(HostInfo^.h_addr_list);
          if (Addr=nil) or (Addr^=nil) then exit;
          tcpiplist.add(StrPas(inet_ntoa(Addr^^)));
          inc(Addr);
          while Addr^<>nil do begin
            tcpiplist.add(StrPas(inet_ntoa(Addr^^)));
            inc(Addr);
          end;
        finally
          WSACleanup;
        end;

     except
     end;

     if assigned(TcpipList) then
     begin
          result:=tcpiplist;
     end;
end;


// Get the DSN lookup name of system
function DNSLookupName(const Name: string): String;
var
  HostEnt: PHostEnt;
  InAddr: TInAddr;
  aData:TWsaData;
  aresult:integer;
  Initialized:Boolean;
begin
  Initialized:=false;
  // try to get the host information by calling the winsock
  // api function; for more info read the Delphi help
  HostEnt := gethostbyname(PChar(Name));
  // check if the call was successful
  aresult:=WSAGetLastError;
  // if we get a result WSANOTINITIALISED we have to initialize the winsock
  if aresult=WSANOTINITIALISED then
  begin
    // intialize the windows socket (winsock) implementation
    // for dns lookup we just need version 1
    // aData is a dummy for us
    WSAStartup(1,aData);
    // now try it again
    HostEnt := gethostbyname(PChar(Name));
    // remember that we initialized it
    Initialized:=true;
  end;
  // initialize the structure to return 0.0.0.0
  FillChar(InAddr, SizeOf(InAddr), 0);
  // if we got a result
  if HostEnt <> nil then
  begin
    // set the fields of InAddr according to the host information
    with InAddr, HostEnt^ do
    begin
      S_un_b.s_b1 := h_addr^[0];
      S_un_b.s_b2 := h_addr^[1];
      S_un_b.s_b3 := h_addr^[2];
      S_un_b.s_b4 := h_addr^[3];
    end;
  end;
  // now we have the four "ip numbers" in in InAddr as Chars
  // a simple typecast will do the rest
  Result := inttostr(byte(Inaddr.S_un_b.s_b1))+'.'+
    inttostr(byte(Inaddr.S_un_b.s_b2))+'.'+
    inttostr(byte(Inaddr.S_un_b.s_b3))+'.'+
    inttostr(byte(Inaddr.S_un_b.s_b4));
  // if we initilized the winsock we have to free it
  if initialized then
    WSACleanup;
  // for everyone who wonders: we are NOT allowed to free the
  // HostEnt structure. See help for more information
end;


// Get a single local IP number
function GetLocalIP: string;
var
  iplist:TStringList;
begin
  result:='';
  iplist:=getlocalips;
  try
    if iplist.count>0 then
      result:=iplist[0];
  finally
    if assigned(iplist) then
     iplist.free;
  end;
end;


// Get a text file and put it into a stringlist for parsing
function HTTPGetTextFile(pWebAddress: String): TStringList;
var
   mTextStrings   : TStringlist;

   IdHTTP1        : TIdHTTP;
   //mData          : TFileStream;
   //mFlag          : Boolean;
   //mHost          : String;
   //mPort          : Integer;
   //mName          : String;
   mFileRequested : String;
   //mFile          : File; // of Byte;
   //mSize          : Longint;
   //mReceived      : Boolean;
   //mErrorLog      : TStringList;

begin
     mTextStrings := TStringlist.Create;

     //mReceived := True;

     IdHTTP1   := TIdHTTP.Create(nil);
     try
         try
            IdHTTP1.ConnectTimeout   := 10000;  //
            mFileRequested := IdHTTP1.Get(pWebAddress);

            mTextStrings.Add(mFileRequested);
         except
            on E: Exception do
            begin
                 //mReceived := False;

                 // Collect an error message
                 GlobMessage := 'HTTPGetFile: Error ('+E.Message +' ('+ IntToStr(E.HelpContext)+') communicating with Web: '+pWebAddress;
                 mTextStrings.Add('ERROR ('+StripOutCRLF(E.Message)+')');
            end;
         end;
         IdHTTP1.Disconnect;

     finally
         IdHTTP1.Free;
     end;

     // Send back the test file, if it was assigned
     //if assigned(mTextStrings) then
     //begin
     result := mTextStrings;
     //end;
end; //  HTTPGetTextFile


// Get a File using HTTP
// HTTP get a single file
function HTTPGetFile(pHost: String; pPort: Integer; pFileName, pFileTo, pFileError: String): Boolean;
var
   IdHTTP1        : TIdHTTP;
   mData          : TFileStream;
   mFlag          : Boolean;
   mHost          : String;
   mPort          : Integer;
   mName          : String;
   mFileRequested : String;
   mFile          : File; // of Byte;
   mSize          : Longint;
   mReceived      : Boolean;
   mErrorLog      : TStringList;

begin
     result         := False;
     mFlag          := True;
     GlobMessage    := '';

     // Get ready to log problems
     mErrorLog   := TStringList.Create;
     try

        // Make sure we have something
        mHost  := pHost;
        if mHost = '' then
        begin
             mHost := 'localhost';
        end;

        // Make sure we have something
        mPort  := pPort;
        if mPort = 0 then
        begin
             mPort := 80;
        end;

        // See if the file exists
        if FileExists(pFileTo) then
        begin
             mFlag := False;

             //GlobDebug   := IntToStr(IOResult);  // This MUST happen before each write
             AssignFile(mFile, pFileTo);
             FileMode := 0;    { read-only access }
             Reset(mFile,1);
             mSize       := FileSize(mFile);
             CloseFile(mFile);

             // Make sure it is at least several bytes
             if mSize < 200 then
             begin
                  DeleteFile(pFileTo);
                  mFlag := True;
             end;
        end;


        // Download it
        if mFlag then
        begin
             mName := ExtractFileName(pFileName); //IdFTP1.DirectoryListing.Items[DirectoryListBox.ItemIndex].FileName;

             mReceived := True;
             mData     := TFileStream.Create(pFileTo, fmCreate);
             IdHTTP1   := TIdHTTP.Create(nil);
             try
                 {
                 IdHTTP1.Host := mHost; //'localhost';
                 IdHTTP1.ConnectTimeout := 30000;  // @@@
                 IdHTTP1.Port := pPort; //5049;
                 IdHTTP1.BoundPort := pPort; //5049;
                 IdHTTP1.Get('http://'+GlobHost+pFileName, mData);
                 IdHTTP1.DisconnectSocket;
                 //IdHTTP1.BoundPort := 0;
                 }
                 try
                    IdHTTP1.ConnectTimeout   := 10000;  //
                    //IdHTTP1.Host             := mHost;  //'localhost';
                    //IdHTTP1.Port             := mPort;  //80;
                    //IdHTTP1.BoundPort        := mPort;  //5049;
                    mFileRequested           := PrefixHTTP(mHost)+':'+IntToStr(mPort)+'/'+pFileName;
                    //mFileRequested           := 'http://localhost:81/'+pFileName;
                    IdHTTP1.Get(mFileRequested, mData);
                 except
                    on E: Exception do
                    begin
                         mReceived := False;

                         // Collect an error message
                         //GlobMessage := 'HTTPGetFile: Error ('+E.Message +' ('+ IntToStr(E.HelpContext)+') communicating with IP:'+IdHTTP1.Host+', Port:'+IntToStr(IdHTTP1.Port);
                         GlobMessage := 'HTTPGetFile: Error ('+E.Message +' ('+ IntToStr(E.HelpContext)+') communicating with IP:'+PrefixHTTP(mHost)+':'+IntToStr(mPort);


                         // Log if we care
                         if pFileError <> '' then
                         begin
                              // Log it please
                              if FileExists(pFileError) then
                              begin
                                   mErrorLog.LoadFromFile(pFileError);
                              end;



                              (*
                              mErrorLog.Add(FormatDateTime('yyyy-mm-dd hh:mm:ss,',Now)+
                                                           'Version: ' +IdHTTP1.Version+','+
                                                           'Host: '    +IdHTTP1.Host+','+
                                                           'Port: '    +IntToStr(IdHTTP1.Port)+','+
                                                           'Request: ' +mFileRequested+','+
                                                           'Error: '   +E.Message +' ('+ IntToStr(E.HelpContext)+')'
                                                           );
                              *)

                              mErrorLog.Add(FormatDateTime('yyyy-mm-dd hh:mm:ss,',Now)+
                                                           'Version: ' +IdHTTP1.GetVersion+','+
                                                           'Host: '    +PrefixHTTP(mHost)+','+
                                                           'Port: '    +IntToStr(mPort)+','+
                                                           'Request: ' +mFileRequested+','+
                                                           'Error: '   +E.Message +' ('+ IntToStr(E.HelpContext)+')'
                                                           );


                              // Save it again
                              mErrorLog.SaveToFile(pFileError);
                         end;
                    end;
                 end;
                 //IdHTTP1.DisconnectSocket;
                 IdHTTP1.Disconnect;

                 //if idHTTP1.ResponseCode > -1 then
                 //begin
             finally
                 IdHTTP1.Free;
                 mData.Free;
             end;

             if mReceived = False then
             begin
                  // An error occured, so clean up
                  DeleteFile(pFileTo);
             end
             else
             begin
                  // Did we get it? If we did, then success
                  if FileExists(pFileTo) then
                  begin
                       result := True;
                  end;
             end;
        end;
     finally
        // Free the log
        mErrorLog.Free;
     end;
end; // HTTPGetFile



// HTTPSendXML - A method to send XML requests
(*
     // Example of how to send an XML...
     mXML             := TStringList.Create;
     try
        mXML.Add(StringReplace('<?xml version=#1.0# encoding=#utf-8#?><REQUEST action=#Add#><PHYSICIAN id=#1212# firstname=#First# lastname=#Last# clientid=#8662723647# clientname=#DEMO# /></REQUEST>', '#', '''', [rfReplaceAll, rfIgnoreCase]));
        mSendResult := HTTPSendXML('http://localhost', 5048, '/MedTek' , mXML);
     finally
        mXML.Free;
     end;
*)
function HTTPSendXML(pHost: String; pPort: Integer; pPage: String; pFileStrings: TStringList): String;
var
   idHttp                : TIdHTTP;
   mResponseStream       : TMemoryStream;
   mLine                 : String;
   mBuffer               : String;
   mLoop                 : Integer;
   mURLRequest           : String;
   mText                 : TStringList;
   mXML                  : TMemoryStream;
begin
     result := '';

     // Define variables
     mText            := TStringList.Create;
     mXML             := TMemoryStream.Create;
     mResponseStream  := TMemoryStream.Create;
     try
        try
           // First line only
           mURLRequest  :=  pHost+':'+IntToStr(pPort)+pPage;

           // Get the XML into a single string
           for mLoop := 0 to pFileStrings.Count - 1 do
           begin
                mBuffer := mBuffer + pFileStrings[mLoop];
           end;
           mXML.WriteBuffer(mBuffer[1], Length(mBuffer));
           //mXML.SaveToFile('e:\junk2.txt');


           // Submit as a request, and recieve response
           idHttp := TIdHTTP.Create(nil);
           try
              //idHttp.Request.ContentType := 'multipart/form-data';
              idHttp.Post(mURLRequest,mXML,mResponseStream);
           finally
              idHttp.Free;
           end;

           // Save the file
           mResponseStream.Position := 0;
           mText.Clear;
           mText.LoadFromStream(mResponseStream);


           // Populate the first line
           for mLoop := 0 to mText.Count -1 do
           begin
                mLine := mLine + Trim(mText[mLoop]);
           end;
           // Return the basic values
           result     := mLine;
        except
           result     := 'ERROR';
        end;

     finally
        mResponseStream.Free;
        mXML.Free;
        mText.Free;
     end;

end; // HTTPSendXML



// Send a file using HTTP
// -- Here is how to do it for NetHub
//
//     mNetCom := '/&~TFSendFile~FileNameToSend.Wav';
//     // or
//     mNetCom := '/&~TFSendScreenCap~'+ExtractFileName(application.exename)+'~'+CalcGetUserName+'~'+CalcGetComputerName+'~'+'ScreenShot.Png';
//
//     HTTPSendFile('http://localhost', 5048, mNetCom, mFileFullPath);
//
(*
   cTFSendFile         = 'TFSendFile';        // [_] http://localhost:78/&~TFSendFile~FileName.Wav
   cTFSendScreenCap    = 'TFSendScreenCap';   // [_] http://localhost:78/&~TFSendScreenCap~Program.exe~UserName~ComputerName~ScreenShotFile.png
*)
function HTTPSendFile(pHost: String; pPort: Integer; pCommand: String; pFileName: String): String;
var
   idHttp                : TIdHTTP;
   mDataStream           : TMemoryStream;
   mResponseStream       : TMemoryStream;
   mLine                 : String;
   mURLRequest           : String;
   mText                 : TStringList;
begin
     result := '';

     // Define variables
     mText            := TStringList.Create;
     mDataStream      := TMemoryStream.Create;
     mResponseStream  := TMemoryStream.Create;
     try
        try
           // First line only
           mURLRequest  :=  pHost+':'+IntToStr(pPort)+pCommand;

           // Submit as a request, and recieve response
           idHttp := TIdHTTP.Create(nil);
           try
             mDataStream.LoadFromFile(pFileName);
             idHttp.Post(mURLRequest,mDataStream,mResponseStream);
           finally
             idHttp.Free;
           end;

           // Save the file
           mResponseStream.Position := 0;
           mText.Clear;
           mText.LoadFromStream(mResponseStream);


           // Populate the first line
           mLine      := mText[0];
           // Return the basic values
           result     := mLine;
        except
           result     := 'ERROR';
        end;

     finally
        mResponseStream.Free;
        mDataStream.Free;
        mText.Free;
     end;

end; // HTTPSendFile


// Submit a simple string
function HTTPSubmitURLRequest(pURLRequest: String): String;
var
   idHttp                : TIdHTTP;
   mDataStream           : TMemoryStream;
   mResponseStream       : TMemoryStream;
   mLine                 : String;
   mText                 : TStringList;
begin
     result := '';

     // Define variables
     mText            := TStringList.Create;
     mDataStream      := TMemoryStream.Create;
     mResponseStream  := TMemoryStream.Create;
     try
        try
           // Submit as a request, and recieve response
           idHttp := TIdHTTP.Create(nil);
           try
             idHttp.ConnectTimeout := 5000;  // Give some time to timeout
             idHttp.Post(pURLRequest,mDataStream,mResponseStream);
           finally
             idHttp.Free;
           end;

           // Save the file
           mResponseStream.Position := 0;
           mText.Clear;
           mText.LoadFromStream(mResponseStream);


           // Populate the first line
           mLine      := mText[0];
           // Return the basic values
           result     := mLine;
        except
           result     := 'ERROR';
        end;

     finally
        mResponseStream.Free;
        mDataStream.Free;
        mText.Free;
     end;
end;


// Send command, and get the response
// eg:  HTTPSendCommandToNetHub(10000, 'http://localhost',5048,'TFPut~.....');
function HTTPSendCommandToNetHub(pTimeOutms: Integer; pHost: String; pPort: Integer; pCommand: String): String;
var
   idHttp                : TIdHTTP;
   mDataStream           : TMemoryStream;
   mLines                : TStringList;
   mHTTPCommand          : String;
begin
     result := '';
     // Get the name of the current file on disk

     // Define variables
     mDataStream := TMemoryStream.Create;
     mLines      := TStringList.Create;
     idHttp := TIdHTTP.Create(nil);
     try
        // Submit as a request, and recieve response
        try
           mHTTPCommand := PrefixHTTP(pHost)+':'+IntToStr(pPort)+'/&~'+pCommand;
           idHttp.ConnectTimeout := pTimeOutms;
           idHttp.Get(mHTTPCommand, mDataStream);
           //idHttp.DisconnectSocket;
           idHttp.Disconnect;

           // Save the file
           mDataStream.Position := 0;
           mLines.LoadFromStream(mDataStream);

           // Just first line should do
           result := mLines[0]; //''; // mDataStream.Lines[0];
         except
           on E: Exception do
           begin
                // Collect an error message
                result := 'ERROR: '+StripOutCRLF(E.Message) +' ('+ IntToStr(E.HelpContext)+') communicating with IP:'+PrefixHTTP(pHost)+', Port:'+IntToStr(pPort);
           end;
         end;

     finally
        idHttp.Free;
        mDataStream.Free;
        mLines.Free;
     end;
end; // SendFileByHTTP





// Get This Program update if it exists
// HTTP a new exename a single file
// - Will arrive as pThisProgram.ex2
(*
    result := HTTPGetProgramUpdate(GlobNetConIP,
                                   GlobNetConPort,
                                   application.exename,
                                   GetVersionInfo(application.exename),
                                   CalcGetUserName,
                                   CalcGetComputerName);

*)
function HTTPGetProgramUpdate(pHost: String; pPort: Integer; pThisProgramExe, pVersion, pUserName, pComputerName: String): String;
const
   cSep               = '~';
   cNew               = '.ex-';
   cAutoUpdateTrigger = 'TFReceiveFile';
var
   IdHTTP1        : TIdHTTP;
   mData          : TFileStream;
   mHost          : String;
   mPort          : Integer;
   mFileRequested : String;
   mReceived      : Boolean;
   mRecord        : String;    // ~Prog~1.0.0~UserName~ComputerName~Ip~
   mUpdateFile    : String;
begin
     result         := '';
     GlobMessage    := '';

     // Make sure we have something
     mHost  := pHost;
     if mHost = '' then
     begin
          mHost := 'localhost';
     end;

     // Make sure we have something
     mPort  := pPort;
     if mPort = 0 then
     begin
          mPort := 80;
     end;

     // Put together the request record
     mRecord := '&~'+
                cAutoUpdateTrigger +
                cSep + ExtractFileName(pThisProgramExe) +
                cSep + pVersion +
                cSep + pUserName +
                cSep + pComputerName +
                cSep + GetLocalIP +
                cSep;
     mRecord := ExtractFileName(pThisProgramExe);

     // The name of the file, should there be an update
     mUpdateFile := ChangeFileExt(pThisProgramExe, cNew);

     // See if the file exists, and remove it
     if FileExists(mUpdateFile) then
     begin
          DeleteFile(mUpdateFile);
     end;

     mReceived := True;
     mData     := TFileStream.Create(mUpdateFile, fmCreate);
     IdHTTP1   := TIdHTTP.Create(nil);
     try
         IdHTTP1.ConnectTimeout   := 5000;
         //IdHTTP1.Host             := mHost;  //'localhost';
         //IdHTTP1.Port             := mPort;  //80;
         mFileRequested           := PrefixHTTP(mHost)+':'+IntToStr(mPort)+'/'+mRecord;
         try
            IdHTTP1.Get(mFileRequested, mData);
         except
            mReceived := False;

            // Collect an error message
            GlobMessage := 'HTTPGetProgramUpdate: Error communicating with IP:'+mHost+', Port:'+IntToStr(mPort);
         end;
         //IdHTTP1.DisconnectSocket;
         IdHTTP1.Disconnect;

     finally
         IdHTTP1.Free;
         mData.Free;
     end;

     if mReceived = False then
     begin
          // An error occured, so clean up
          DeleteFile(mUpdateFile);
     end
     else
     begin
          // Did we get it? If we did, then success
          if FileExists(mUpdateFile) then
          begin
               result := mUpdateFile;
          end;
     end;

end; // HTTPGetProgramUpdate


// Log back to the main server
function HTTPLog(pHost: String; pPort: Integer; pLogCode, pLogMessage: String): Boolean;
const
   cSep='~';
var
   IdHTTP1        : TIdHTTP;
   mDataStream    : TMemoryStream;
   mHost          : String;
   mPort          : Integer;
   mFileRequested : String;
   mRecord        : String;    // ~Prog~1.0.0~UserName~ComputerName~Ip~

   mList          : TStringList;

begin
     result         := True;
     GlobMessage    := '';

     mList  := TStringList.Create;
     try
        // Make sure we have something
        mHost  := pHost;
        if mHost = '' then
        begin
             mHost := cLocalHost;
        end;

        // Make sure we have something
        mPort  := pPort;
        if mPort = 0 then
        begin
             mPort := 80;
        end;

        // Put together the request record
        mRecord := '/&~TFLog~' + pLogCode +
                   cSep + pLogMessage +
                   cSep + GetLocalIP;

        mList.Add(mRecord);

        mDataStream := TMemoryStream.Create;
        IdHTTP1     := TIdHTTP.Create(nil);
        try
            IdHTTP1.ConnectTimeout   := 3000;
            //IdHTTP1.Host             := mHost;  //'localhost';
            //IdHTTP1.Port             := mPort;  //80;
            mFileRequested           := PrefixHTTP(mHost)+':'+IntToStr(mPort)+mRecord;
            try
               //IdHTTP1.Get(mFileRequested, mDataStream);
               IdHTTP1.Post(mFileRequested, mList);   // This one might be less intensive .. go with this.
            except
               on E: Exception do
               begin
                    result    := False;
                    // Collect an error message
                    GlobMessage := 'HTTPLog: '+StripOutCRLF(E.Message) +' ('+ IntToStr(E.HelpContext)+') communicating with IP:'+PrefixHTTP(mHost)+', Port:'+IntToStr(mPort);
               end;

            end;
            //IdHTTP1.DisconnectSocket;
            IdHTTP1.Disconnect;

        finally
            mDataStream.Free;
            IdHTTP1.Free;
        end;
     finally
        mList.Free;
     end;

     if GlobMessage <> '' then
     begin
          raise Exception.CreateFmt('%s', [GlobMessage]);
     end;
end; // HTTPLog


// Ensure an IP is a Host
function PrefixHTTP(pHost: String): String;
const
   cHTTP   = 'http://';
   cHTTPS  = 'https://';
var
   mHost   : String;
begin
     mHost := lowercase(pHost);
     if (pos(cHTTP, mHost) > 0) then
     begin
          result := cHTTP + Copy(pHost,Length(cHTTP)+1,9999);
     end
     else if (pos(cHTTPS, mHost) > 0) then
     begin
          result := cHTTPS + Copy(pHost,Length(cHTTPS)+1,9999);
     end
     else
     begin
          // Prepend an HTTPS
          result := cHTTP + pHost;
     end;
end; // PrefixHTTP


// Provide last error.  Blank if not one
function HTTPLastErrorMessage: String;
begin
     result := GlobMessage;
end;


end.