unit VoiceConvertUnit;

   {/* standard wave file header */
   /****************************************************************
   * 'RIFF'
   * length in bytes of all that follows(long)
   * 'WAVE'
   * 'fmt '
   * length in bytes of the format block = 16(long)
   * format = 1(int)
   * number of channels = 1(int) - mono
   * sample rate(long)
   * bytes per second during play(long)
   * bytes per sample = 2(int)
   * bits per sample = 16(int)
   * 'data'
   * length in bytes of the data block(long)
   ****************************************************************/}

(*
The Format Chunk
The Format (fmt) chunk describes fundamental parameters of
the waveform data such as sample rate, bit resolution, and
how many channels of digital audio are stored in the WAVE.

#define FormatID 'fmt '   /* chunkID for Format Chunk.
NOTE: There is a space at the end of this ID. */

typedef struct {                        //WAVEfmt_  = 8
  ID             chunkID;               //
  long           chunkSize;             //          = 8

  short          wFormatTag;            //          = 2
  unsigned short wChannels;
  unsigned long  dwSamplesPerSec;
  unsigned long  dwAvgBytesPerSec;
  unsigned short wBlockAlign;
  unsigned short wBitsPerSample;

/* Note: there may be additional fields here, depending upon wFormatTag. */

} FormatChunk;

The ID is always "fmt ". The chunkSize field is the number of bytes in
the chunk. This does not include the 8 bytes used by ID and Size fields.
For the Format Chunk, chunkSize may vary according to what "format" of
WAVE file is specified (ie, depends upon the value of wFormatTag).

WAVE data may be stored without compression, in which case the sample
points are stored as described in Sample Points and Sample Frames.
Alternately, different forms of compression may be used when storing the
sound data in the Data chunk. With compression, each sample point may
take a differing number of bytes to store. The wFormatTag indicates
whether compression is used when storing the data.

If compression is used (ie, WFormatTag is some value other than 1), then
there will be additional fields appended to the Format chunk which give
needed information for a program wishing to retrieve and decompress that
stored data. The first such additional field will be an unsigned short
that indicates how many more bytes have been appended (after this
unsigned short). Furthermore, compressed formats must have a Fact chunk
which contains an unsigned long indicating the size (in sample points)
of the waveform after it has been decompressed. There are (too) many
compressed formats. Details about them can be gotten from Microsoft's
web site.

If no compression is used (ie, wFormatTag = 1), then there are no
further fields.

The wChannels field contains the number of audio channels for the
sound. A value of:
1 means monophonic sound,
2 means stereo,
4 means four channel sound, etc.

Any number of audio channels may be represented.
For multichannel sounds, single sample points from each channel
are interleaved. A set of interleaved sample points is called a sample frame.
*)

{ EXAMPLE CODE:

// Convert Wav File
procedure TMainForm.FileConvertCreate(pFileNameFrom, pFileNameTo: String);
var
   mFileMode: playmodes;
   mMessage: String;
begin
     // Check the file type.
     // Then convert it
     if UpperCase(ExtractFileExt(pFileNameFrom)) = '.VOX' then
     begin
          // Vox
          // Convert the file
          VoiceConvertUnit.VoxtoWav(pFileNameFrom, pFileNameTo, 8000, 8000);
     end
     else
     begin
          // Wav
          // Convert the file
          //with VoiceConvertUnit do
          //begin
               VoiceConvertUnit.WAVFormatCheck(pFileNameFrom,mFileMode);
               //voxtype          := mFileMode; //VT_ADPCM;
               VoiceConvertUnit.SetPropStoredMode       (VT_ADPCM); //mFileMode;  //VT_ADPCM,VT_PCM,VT_ALAW,VT_MULAW
               VoiceConvertUnit.SetPropSampleDownFilter (True);
               VoiceConvertUnit.SetPropWaveOut          (True);
               mMessage         := VoiceConvertUnit.WavtoVox(pFileNameFrom, pFileNameTo, 8000);
          //end;

          // See if there was a problem
          if mMessage <> '' then
          begin
               MessageDlg('Error: Failed to convert '+pFileNameFrom,mtError,[mbOk],0);
          end;

     end;
end;  // FileConvertCreate

}

interface

{$WARN UNIT_PLATFORM OFF}

{$ifdef VER140} // Delphi 6
  {$WARN SYMBOL_PLATFORM OFF}
{$endif}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, mmsystem, Math;

// Voice Format Definitions
type playmodes = (VT_ADPCM,VT_PCM,VT_ALAW,VT_MULAW);
type pbytearray=^tbytearray;
     tbytearray=array[0..100000000] of byte;
type psmallintarray=^tsmallintarray;
     tsmallintarray=array[0..100000000] of smallint;


    procedure InData(sender: TObject; data: PChar; size: Integer);

    function  play(nome : string): boolean;
    function  startrecording(nome : string; mode : playmodes): boolean;
    procedure stoprecording;
    procedure PCM16toADPCM(frombuffer : psmallintarray ; tobuffer : pbytearray ; var sspointer,xp :integer; count : integer);
    procedure PCM16toALAW(frombuffer : psmallintarray ; tobuffer : pbytearray ; count : integer);
    procedure PCM16toMULAW(frombuffer : psmallintarray ; tobuffer : pbytearray ; count : integer);
    procedure ADPCMtoPCM16(frombuffer : pbytearray ;  tobuffer : psmallintarray; var sspointer,xp :integer; count : integer);
    procedure ALAWtoPCM16(frombuffer : pbytearray ;  tobuffer : psmallintarray; count : integer);
    procedure MULAWtoPCM16(frombuffer : pbytearray ;  tobuffer : psmallintarray; count : integer);
    procedure PCM16toPCM8(frombuffer: psmallintarray ; tobuffer: pbytearray; count : integer);
    procedure PCM8toPCM16(frombuffer: pbytearray ; tobuffer: psmallintarray; count : integer);
    function  RateConvert(buffer: psmallintarray ; count : integer ; fromfreq,tofreq: integer) : integer;
    function  WAVFormatCheck(nome: string;var kind :playmodes) : boolean;
    function  WavtoVox(nomein,nomeout : string;voxrate : integer): String;
    function  VoxtoWav(nomein,nomeout : string; infreq, outfreq : integer ): String;

    procedure SetPropStoredMode       (pPropStoredMode       :playmodes);
    procedure SetPropVoxType          (pPropVoxType          :playmodes);
    procedure SetPropWaveOut          (pPropWaveOut          :boolean);
    procedure SetPropSampleDownFilter (pPropSampleDownFilter :boolean);





    // property PlayRate: integer read fplayrate write setplayrate default 8000;
    //property VoxType : playmodes read fvoxtype  write fvoxtype;       // PropVoxType
    //property WavType : playmodes read fstoredmode write fstoredmode;  //
    //property waveout16bit : boolean read f16bitwaveout write f16bitwaveout;  // PropWaveOut
    //property downsamplefilter : boolean read fdownsamplefilter write fdownsamplefilter;   // PropSampleDownFilter
    // property WaveOut : TEnhWaveOut read fwaveout write fwaveout;
    // property WaveIn  : TEnhWaveIn read  fwavein write fwavein;

implementation

var
   // Properties in Unit
   PropStoredMode        : playmodes;
   PropVoxType           : playmodes;
   PropWaveOut           : boolean;
   PropSampleDownFilter  : boolean;

   //
   waslast               : boolean;
   fo                    : file;

   ADPCMbuffer:array[0..5000] of byte;
   PCM8buffer:array[0..10000] of byte;
   PCM16buffer:array[0..10000] of smallint;
   storedXP,storedSSpointer : integer;
   MLN : array[0..7] of integer = (-1,-1,-1,-1,+2,+4,+6,+8);
   SS : array[0..48] of word =    (16,17,19,21,23,25,28,31,34,37,41,45,50,55,
                                   60,66,73,80,88,97,107,118,130,143,157,173,190,
                                   209,230,253,279,307,337,371,408,449,494,544,
                                   598,658,724,796,876,963,1060,1166,1282,1411,1552);

   //* alaw decode table */

   alaw: array[0..255] of integer=
   (-688,-656,-752,-720,-560,-528,-624,-592,-944,-912,-1008,-976,
   -816,-784,-880,-848,-344,-328,-376,-360,-280,-264,-312,-296,
   -472,-456,-504,-488,-408,-392,-440,-424,-2752,-2624,-3008,-2880,
   -2240,-2112,-2496,-2368,-3776,-3648,-4032,-3904,-3264,-3136,
   -3520,-3392,-1376,-1312,-1504,-1440,-1120,-1056,-1248,-1184,
   -1888,-1824,-2016,-1952,-1632,-1568,-1760,-1696,-43,-41,-47,-45,
   -35,-33,-39,-37,-59,-57,-63,-61,-51,-49,-55,-53,-11,-9,-15,
   -13,-3,-1,-7,-5,-27,-25,-31,-29,-19,-17,-23,-21,-172,-164,
   -188,-180,-140,-132,-156,-148,-236,-228,-252,-244,-204,-196,
   -220,-212,-86,-82,-94,-90,-70,-66,-78,-74,-118,-114,-126,-122,
   -102,-98,-110,-106,688,656,752,720,560,528,624,592,944,912,
   1008,976,816,784,880,848,344,328,376,360,280,264,312,296,472,
   456,504,488,408,392,440,424,2752,2624,3008,2880,2240,2112,
   2496,2368,3776,3648,4032,3904,3264,3136,3520,3392,1376,1312,
   1504,1440,1120,1056,1248,1184,1888,1824,2016,1952,1632,1568,
   1760,1696,43,41,47,45,35,33,39,37,59,57,63,61,51,49,55,53,
   11,9,15,13,3,1,7,5,27,25,31,29,19,17,23,21,172,164,188,
   180,140,132,156,148,236,228,252,244,204,196,220,212,86,82,
   94,90,70,66,78,74,118,114,126,122,102,98,110,106);

   //* mulaw decode table */
   mulaw: array[0..255] of integer=
   (-8031,-7775,-7519,-7263,-7007,-6751,-6495,-6239,-5983,-5727,-5471,
   -5215,-4959,-4703,-4447,-4191,-3999,-3871,-3743,-3615,-3487,
   -3359,-3231,-3103,-2975,-2847,-2719,-2591,-2463,-2335,-2207,
   -2079,-1983,-1919,-1855,-1791,-1727,-1663,-1599,-1535,-1471,
   -1407,-1343,-1279,-1215,-1151,-1087,-1023,-975,-943,-911,-879,
   -847,-815,-783,-751,-719,-687,-655,-623,-591,-559,-527,-495,
   -471,-455,-439,-423,-407,-391,-375,-359,-343,-327,-311,-295,
   -279,-263,-247,-231,-219,-211,-203,-195,-187,-179,-171,-163,
   -155,-147,-139,-131,-123,-115,-107,-99,-93,-89,-85,-81,-77,
   -73,-69,-65,-61,-57,-53,-49,-45,-41,-37,-33,-30,-28,-26,-24,
   -22,-20,-18,-16,-14,-12,-10,-8,-6,-4,-2,0,8031,7775,7519,
   7263,7007,6751,6495,6239,5983,5727,5471,5215,4959,4703,4447,
   4191,3999,3871,3743,3615,3487,3359,3231,3103,2975,2847,2719,
   2591,2463,2335,2207,2079,1983,1919,1855,1791,1727,1663,1599,
   1535,1471,1407,1343,1279,1215,1151,1087,1023,975,943,911,879,
   847,815,783,751,719,687,655,623,591,559,527,495,471,455,439,
   423,407,391,375,359,343,327,311,295,279,263,247,231,219,211,
   203,195,187,179,171,163,155,147,139,131,123,115,107,99,93,
   89,85,81,77,73,69,65,61,57,53,49,45,41,37,33,30,28,26,24,
   22,20,18,16,14,12,10,8,6,4,2,0);

   {/* standard wave file header */
   /****************************************************************
   * 'RIFF'
   * length in bytes of all that follows(long)
   * 'WAVE'
   * 'fmt '
   * length in bytes of the format block = 16(long)
   * format = 1(int)
   * number of channels = 1(int) - mono
   * sample rate(long)
   * bytes per second during play(long)
   * bytes per sample = 2(int)
   * bits per sample = 16(int)
   * 'data'
   * length in bytes of the data block(long)
   ****************************************************************/}
   wavehdr: array [0..10] of longint = ($46464952,$ffffffdb,$45564157,$20746d66,16,$10001,6000,
                      6000,$80001,$61746164,-1);


procedure SetPropStoredMode       (pPropStoredMode       :playmodes);
begin
     PropStoredMode               := pPropStoredMode;
end;

procedure SetPropVoxType          (pPropVoxType          :playmodes);
begin
     PropVoxType                  := pPropVoxType;
end;

procedure SetPropWaveOut          (pPropWaveOut          :boolean);
begin
     PropWaveOut                  := pPropWaveOut;
end;

procedure SetPropSampleDownFilter (pPropSampleDownFilter :boolean);
begin
     PropSampleDownFilter         := pPropSampleDownFilter;
end;


procedure InData(sender: TObject; data: PChar;
  size: Integer);
begin
     size:=size and $fffffffe; {al pari inferiore  }
     if PropStoredMode=VT_ADPCM then
     begin
          Pcm16ToAdpcm(pointer(data),@ADPCMbuffer, storedSSpointer, storedXp, size);
          blockwrite(fo,ADPCMbuffer,size div 2);
     end else blockwrite(fo,data^,size);
     if waslast then closefile(fo);
end;

procedure stoprecording;
begin
     waslast:=true;
     //if fwavein.isopen then fwavein.close;
end;

function startrecording(nome : string; mode : playmodes): boolean;
begin
     result := True;
    {if assigned(fwavein) then
      begin
      storedsspointer:=0;
      storedxp:=2048;
      assignfile(fo,nome);
      if ioresult<>0 then
        begin
        result:=false;
        exit;
        end;
      rewrite(fo,1);
      if ioresult<>0 then
        begin
        result:=false;
        exit;
        end;
      PropStoredMode:=mode;
      fwavein.ondata:=indata;
      fwavein.samplebits:=8;
      fwavein.buffersize:=2048;
      waslast:=false;
      fwavein.open;
      end else showmessage('Must assign WaveIn property');
    result:=assigned(fwavein);}
end;


procedure MULAWtoPCM16(frombuffer: pbytearray ;tobuffer : psmallintarray; count : integer);
var
   offset: integer;
begin
     for offset:=0 to count-1 do
       tobuffer[offset]:=mulaw[frombuffer[offset]] * 4 ;  { div se voglio 8 bit altrimenti *4}
end;


procedure ALAWtoPCM16(frombuffer : pbytearray ; tobuffer : psmallintarray ; count : integer);
var
   offset: integer;
begin
     for offset:=0 to count-1 do
       tobuffer[offset]:=alaw[frombuffer[offset]] * 8 ;  { div se voglio 8 bit altrimenti *8}
end;


procedure ADPCMtoPCM16(frombuffer : pbytearray ;tobuffer : psmallintarray; var sspointer,xp :integer; count : integer);
var
   counter: integer;
   dn: integer;
   sample: word;
   ssvalue:integer;
begin
     ssvalue:=ss[sspointer];
     for counter:=0 to ((count*2)-1)  do
     begin
       if (counter mod 2) = 0 then
         sample:=frombuffer[counter div 2] and $f0 div 16
                              else
         sample:=frombuffer[counter div 2] and $f;
       dn := (ssvalue*((sample and $4) div 4))+((ssvalue div 2)*((sample and $2) div 2))
              +((ssvalue div 4)*(sample and $1))+ (ssvalue div 8);
       inc(sspointer,MLN[sample and $7]);
       if sspointer<0 then sspointer:=0;
       if sspointer>48 then sspointer:=48;
       ssvalue:=ss[sspointer];
       if (sample and $8) = 8 then dn:=-dn;
       xp:=xp+dn;
       if xp>4095 then xp:=4095;
       if xp<0 then xp:=0;
      // tobuffer[counter]:=byte(xp div 16);
       tobuffer[counter]:=smallint((xp-2048)*16);
     end;
end;


procedure PCM16toALAW(frombuffer: psmallintarray; tobuffer : pbytearray; count : integer);
var
    offset : integer;
    sign : integer;
    sig : smallint;
begin
     for offset:=0 to count-1 do
     begin
          sig:=frombuffer[offset] div 8;
          if sig>=0 then sign:=128
                   else begin
                        sig:=-sig;
                        sign:=0;
                        end;
          if sig<64 then tobuffer[offset]:= (sign+(sig div 2)) xor $55
                    else
            if sig<128 then tobuffer[offset]:= (sign+(sig div 4)+16) xor $55
                      else
              if sig<256 then tobuffer[offset]:= (sign+(sig div 8)+32) xor $55
                    else
                if sig<512 then tobuffer[offset]:= (sign+(sig div 16)+48) xor $55
                      else
                  if sig<1024 then tobuffer[offset]:= (sign+(sig div 32)+64) xor $55
                        else
                    if sig<2048 then tobuffer[offset]:= (sign+(sig div 64)+80) xor $55
                          else
                      if sig<4096 then tobuffer[offset]:= (sign+(sig div 128)+96) xor $55
                            else tobuffer[offset]:= (sign+127) xor $55;
     end;
end;


procedure PCM16toPCM8(frombuffer: psmallintarray ; tobuffer: pbytearray; count : integer);
var
   a: integer;
begin
     for a:=0 to count-1 do tobuffer[a]:=byte((longint(frombuffer[a])+32768) div 256);
end;


procedure PCM8toPCM16(frombuffer: pbytearray ; tobuffer: psmallintarray; count : integer);
var
   a: integer;
begin
     for a:=0 to count-1 do tobuffer[a]:=(smallint(frombuffer[a])-128)*256;
end;



procedure PCM16toMULAW(frombuffer: psmallintarray ; tobuffer: pbytearray; count : integer);
var
    offset : integer;
    sign : integer;
    sig : smallint;
begin
     for offset:=0 to count-1 do
     begin
          sig:=frombuffer[offset] div 4;
          if sig>0 then sign:=255
                   else begin
                        sig:=-sig;
                        sign:=127;
                        end;
          if sig<32 then tobuffer[offset]:= sign-(sig div 2)
                    else
            if sig<96 then tobuffer[offset]:= sign-(sig div 4)-8
                      else
              if sig<224 then tobuffer[offset]:= sign-(sig div 8)-20
                    else
                if sig<480 then tobuffer[offset]:= sign-(sig div 16)-34
                      else
                  if sig<992 then tobuffer[offset]:= sign-(sig div 32)-49
                        else
                    if sig<2016 then tobuffer[offset]:= sign-((sig+32) div 64)-64
                          else
                      if sig<4064 then tobuffer[offset]:= sign-((sig+32) div 128)-80
                            else
                        if sig<8160 then tobuffer[offset]:= sign-((sig+32) div 256)-96
                              else tobuffer[offset]:= sign-127;
     end;
end;


procedure PCM16toADPCM(frombuffer: psmallintarray ;tobuffer : pbytearray; var sspointer,xp :integer; count : integer);
var offset : integer;
    encoded : byte;

function adpcm( csig : integer) : byte;
var
   diff,step: integer;
   encodedato : byte;
begin
     step:=ss[sspointer];
     if (csig>4095) then csig:=4095;
     if (csig<0) then csig:=0;
     diff:=csig-Xp;
     if diff<0 then
     begin
       encodedato:=8;
       diff:=-diff;
     end     else encodedato:=0;
     if diff>=step then
     begin
       inc(encodedato,4);
       dec(diff,step)
     end;
     step:=step div 2;
     if diff>=step then
     begin
       inc(encodedato,2);
       dec(diff,step)
     end;
     step:=step div 2;
     if diff>=step then
       inc(encodedato,1);
     step:=ss[sspointer];
     diff := ((step)*((encodedato and $4) div 4))+((step div 2)*((encodedato and $2) div 2))
                 +((step div 4)*(encodedato and $1))+ (step div 8);
     if (encodedato and $8) = 8 then diff:=-diff;
     inc(sspointer,MLN[encodedato and $7]);
     if sspointer<0 then sspointer:=0;
     if sspointer>48 then sspointer:=48;
     xp:=xp+diff;
     adpcm:=encodedato;
end;

begin
   for offset:=0 to ((count div 2)-1) do
   begin
        encoded:=byte(adpcm((frombuffer[offset*2] div 16)+2048)*16);
        inc(encoded,byte(adpcm((frombuffer[offset*2+1] div 16)+2048)) );
        tobuffer[offset]:=encoded;
   end;
end;


// Figure out the file format
function WAVFormatCheck(nome: string;var kind :playmodes) : boolean;
var
    fi: file;

    letti,
    a,
    mLoop,
    mType,
    mTypeCompression,
    totale: integer;

    riffbuff : array[1..4] of char;
    mADPCMbuffer: array[0..2047] of Byte;
begin
     assignfile(fi,nome);
     filemode:=0;
     reset(fi,1);
     totale:=0;
     blockread(fi,mADPCMbuffer,1024,letti);

     for mLoop:=1 to 4 do
     begin
          riffbuff[mLoop]:=char(mADPCMbuffer[mLoop-1]);
     end;

     result:=false;

     if riffbuff = 'RIFF' then
     begin
          // look for the WAVEfmt, then the next byte following is the format
          if letti > 200 then
          begin
               for a:=0 to 200 do
               begin
                    for mLoop:=1 to 4 do
                    begin
                         riffbuff[mLoop]:=char(mADPCMbuffer[a+mLoop-1]);
                    end;

                    //WAVEfmt_  = 8
                    //          = 8
                    //          = 2
                   // VT_ADPCM,
                   // VT_PCM,  = Windows
                   // VT_ALAW,
                   // VT_MULAW = mu Law


                   (*
                   typedef struct {
                     ID             chunkID;         4
                     long           chunkSize;       4

                     short          wFormatTag;      2
                     unsigned short wChannels;
                     unsigned long  dwSamplesPerSec;
                     unsigned long  dwAvgBytesPerSec;
                     unsigned short wBlockAlign;
                     unsigned short wBitsPerSample;
                   *)


                    if riffbuff = 'WAVE' then
                    begin
                         (*
                         |*|     4 bytes 'RIFF'
                         |*|     4 bytes <length>
                         |*|     4 bytes 'WAVE'
                         |*|     4 bytes 'fmt '
                         |*|     4 bytes  <length>     ; 10h - length of 'data' block
                         |*|     2 bytes  01      ; format tag
                         |*|     2 bytes  01      ; channels (1=mono, 2=stereo)
                         |*|     4 bytes  xxxx     ; samples per second
                         |*|     4 bytes  xxxx     ; average samples per second
                         |*|     2 bytes  01      ; block alignment
                         |*|     2 bytes  08      ; bits per sample
                         |*|     4 bytes 'data'
                         |*|     4 bytes <length>
                         |*|       bytes <sample data>
                         *)

                         // Load information about the format
                         mType            := mADPCMbuffer[a+12];  // Get the type
                         mTypeCompression := mADPCMbuffer[a+13];  // Get the type

                         if mType = 1 then  // 0
                         begin
                              kind:=VT_ADPCM;  // Windows PCM
                         end
                         else
                         begin
                              kind:=VT_ADPCM;

                              (*
                              if mTypeCompression = 1 then
                              begin
                                   kind:=VT_MULAW;
                              end
                              else
                              if mTypeCompression = 2 then
                              begin
                                   kind:=VT_ALAW;
                              end
                              else
                              if mTypeCompression = 3 then
                              begin
                                   kind:=VT_ADPCM;
                              end
                              *)
                         end;

                         break;
                    end;
               end
          end;
     end
     else
     begin
          if letti=1024 then
          begin
               for a:=0 to letti-1 do
               begin
                    inc(totale,abs(mADPCMbuffer[a]-mADPCMbuffer[a+1]));
               end
          end
          else
          begin
               totale:=maxint;
          end;

          if totale > 40000 then
          begin
               kind:=VT_ADPCM
          end
          else
          begin
               kind:=VT_PCM;
          end;

          result:=true;
     end;
     closefile(fi);
end;


function play(nome : string) :boolean;
(*
//var
   //fi,fo:file;
   //lettitot:longint;
   //tipo : playmodes;
   //letti :integer;
   //sspointer: integer;
*)
begin
     result := True;
     {result:=true;
     if not fwaveout.isopen then fwaveout.open;
     if fwaveout.isplaying then fwaveout.reset;
     if fileexists(nome) then
     begin
       tipo:=PropVoxType;
       if tipo=VT_EURISTIC then
       begin
         if not euristiccheck(nome,tipo) then
         begin
            result:=false;
            exit;
         end;
       end;
       assignfile(fi,nome);
       filemode:=2;
       reset(fi,1);
       sspointer:=0;
       xp:=2048;
       repeat
         blockread(fi,ADPCMbuffer,1024,letti);
         if tipo=VT_ADPCM then
           begin
           ADPCMtoPCM(@ADPCMbuffer,@PCMbuffer,sspointer,xp,letti);
           fwaveout.playback(@PCMbuffer,letti*2);
           end else fwaveout.playback(@ADPCMbuffer,letti);
       until letti<>1024;
       closefile(fi);
     end else result:=false;}
end;


function WavtoVox(nomein,nomeout : string;voxrate : integer): String;
var
    handle : hmmio;
    infoparent : tmmckinfo;
    infosub    : tmmckinfo;
    formato    : twaveformatex;
    fmtsize    : integer;
    xp,sspointer,letti
               : integer;
    fo         : file;
    toberead,readnow
               : integer;
    samplesread: integer;

begin
     result := '';

     handle:=mmioOpen(pchar(nomein), nil, MMIO_READ or MMIO_ALLOCBUF);
     if (handle=0) then
       begin
            result := 'Could not open Wave file';
            exit;
       end;

     {/*
      * Locate a "RIFF" chunk with a "WAVE" form type
      * to make sure the file is a WAVE file.
      */    }
     infoParent.fccType:=ord('W')+ord('A')*$100+ord('V')*$10000+ord('E')*$1000000;
     if mmioDescend(handle, @infoParent, nil, MMIO_FINDRIFF)<>MMSYSERR_NOERROR then
     begin
         result := 'This is not a Wave file';
         mmioClose(handle, 0);
         exit
     end;

     {    /*
      * Find the "fmt " chunk (form type "fmt "); it must be
      * a subchunk of the "RIFF" parent chunk.
      */   }
     infoSub.ckid:=ord('f')+ord('m')*$100+ord('t')*$10000+ord(' ')*$1000000;

     if mmioDescend(handle, @infoSub, @infoParent,MMIO_FINDCHUNK)<>MMSYSERR_NOERROR then
     begin
         result := 'Wave file has no format';
         mmioClose(handle, 0);
         exit;
     end;

     {    /*
      * Get the size of the "fmt " chunk--allocate and lock memory for it.
      */  }
     FmtSize:= infoSub.cksize;
     //  /* Read the "fmt " chunk. */
     if (mmioRead(handle, @formato, FmtSize)<>FmtSize) then  //per me fa un pos chifo..
     begin
          result := 'Failed to read format';
          mmioClose(handle, 0);
          exit;
     end;

     //    /* Ascend out of the "fmt " subchunk. */
     mmioAscend(handle, @infoSub, 0);

     { /*
      * Find the data subchunk. The current file position
      * should be at the beginning of the data chunk.
      */}
     infoSub.ckid := ord('d')+ord('a')*$100+ord('t')*$10000+ord('a')*$1000000;
     if (mmioDescend(handle, @infosub, @infoparent,MMIO_FINDCHUNK))<>MMSYSERR_NOERROR then
     begin
          result := 'Cannot find data';
          mmioClose(handle, 0);
          exit;
     end;

     if (infoSub.cksize=0)  then
     begin
          result := 'Empty recording';
          exit;
     end;

     toberead:=infosub.cksize;

     assignfile(fo,nomeout);
     if ioresult<>0 then
     begin
         sysutils.beep;
         exit;
     end;
     rewrite(fo,1);
     if ioresult<>0 then
     begin
         sysutils.beep;
         exit;
     end;
     sspointer:=0;
     xp:=2048;

     repeat
          if toberead >2048*(formato.wbitspersample div 8)
                            then readnow:=2048*(formato.wbitspersample div 8)
                            else readnow:=toberead;

          if formato.wbitspersample=8 then
          begin
            letti:=mmioRead(handle, @PCM8buffer,readnow);
            Pcm8topcm16(@PCM8buffer,@Pcm16buffer,letti);
          end else letti:=mmioRead(handle, @PCM16buffer,readnow);
          samplesread:=letti div (formato.wbitspersample div 8);

          if (formato.Nsamplespersec<>voxrate) and (voxrate<>0) then
          begin
            samplesread:=RateConvert(@pcm16buffer,samplesread,formato.Nsamplespersec,voxrate);
          end;

          if readnow<>letti then toberead:=0
                            else dec(toberead,readnow);

          case PropStoredMode of
            vt_pcm:  begin
                     Pcm16topcm8(@PCM16buffer,@Pcm8buffer,samplesread);
                     blockwrite(fo,PCM8buffer,samplesread);
                     end;
            Vt_adpcm:
                     begin
                     Pcm16ToAdpcm(@PCM16buffer,@ADPCMbuffer, SSpointer, Xp, samplesread);
                     blockwrite(fo,ADPCMbuffer,samplesread div 2);
                     end;
            Vt_alaw:
                     begin
                     Pcm16ToAlaw(@PCM16buffer,@PCM8buffer, samplesread);
                     blockwrite(fo,PCM8buffer,samplesread);
                     end;
            Vt_mulaw:
                     begin
                     Pcm16Tomulaw(@PCM16buffer,@PCM8buffer, samplesread);
                     blockwrite(fo,PCM8buffer,samplesread);
                     end;
          end;
     until toberead=0;

     closefile(fo);
     mmioClose(handle, 0);
end;



function RateConvert(buffer: psmallintarray ;
                      count : integer ;
                      fromfreq,tofreq: integer ) : integer;


  function power2(i:integer):integer;
  const powers:array[0..20] of integer=(1,2,4,8,16,32,64,128,256,512,1024,
                                        2048,4096,8192,16384,32768,65536,
                                        131072,262144,524288,1048576);
begin
     power2:=powers[i];
end;


{From "Image Processing" by Jan Teuber}
procedure fft(var fr,fi:array of single;lnN:integer;sign:integer);
var
    // nm1,
    nd2:integer;
    i,j,k,l,le,le1,ip:integer;
    s,ur,ur1,ui,wr,wi,tr,ti:single;
    n:integer;
    divN:single;
begin
  n  :=power2(lnN);
  nd2:=N div 2;
  //nm1:=n-1;
  j  :=1;
  for i:=1 to N-1 do
  begin
    if i<j then
    begin
      s:=fr[i]; fr[i]:=fr[j]; fr[j]:=s;
      s:=fi[i]; fi[i]:=fi[j]; fi[j]:=s;
    end;  { if }
    k:=nd2;
    while k<j do
    begin
      j:=j-k;
      k:=k div 2;
    end;  { while }
    j:=j+k;
  end;
  for l:=1 to lnN do
  begin
    le :=Power2(l);
    le1:=le div 2;
    ur :=1;
    ui :=0;
    wr :=cos(pi/le1);
    wi :=sign*sin(pi/le1);
    for j:=1 to le1 do
    begin
      i:=j;
      while (i<=N) do
      begin
        ip:=i+le1;
        tr:=fr[ip]*ur-fi[ip]*ui;
        ti:=fr[ip]*ui+fi[ip]*ur;
        fr[ip]:=fr[i]-tr;
        fi[ip]:=fi[i]-ti;
        fr[i] :=fr[i]+tr;
        fi[i] :=fi[i]+ti;
        i := i + le;
      end;  { while }
      ur1:=ur*wr-ui*wi;
      ui :=ur*wi+ui*wr;
      ur :=ur1;
    end;  { for j }
  end;  { for l }
  divN:=1/sqrt(n);
  for i:=1 to N do
  begin
    fr[i]:=fr[i]*divN;
    fi[i]:=fi[i]*divN;
  end;  { for i }
end;
// end FFT

var
   sr,si :array[0..8191] of single;
   countout,a,incr,endmargin : integer;
   pos: single;
   n,logn: integer;
   keep: integer;
   //interpolate: boolean;
begin
     countout:=round(count/fromfreq*tofreq);

     if countout>count then
     begin
          //upsampling then interpolate only
          incr:=-1;
          a:=countout-1;
          endmargin:=0;
     end else
     begin
          // downsampling, filter and interpolate
          if PropSampleDownFilter then
          begin
            logn:=ceil(log2(count));
            N:=power2(logn);
            fillchar(si,sizeof(si),0);
            fillchar(sr,sizeof(sr),0);
            for a:=0 to count-1 do
              sr[a]:=buffer[a];
            fft(sr,si,logN,-1); // to frequency domain
            keep:=countout div 2;
            for a := 1+keep to N-keep do
            begin      //low pass AntiAlias filter;
              sr[a] := 0;
              si[a] := 0;
            end;
            fft(sr,si,logN,1); // back to time domain
            for a:=0 to count-1 do
              buffer[a]:=round(sr[a]);
          end;
          incr:=1;
          a:=0;
          endmargin:=countout-1;
     end;

     while a<>endmargin do
     begin
          pos:=a*count/countout;
          buffer[a]:=round((1-frac(pos))*buffer[trunc(pos)]+frac(pos)*buffer[trunc(pos)+1]);
          inc(a,incr);
     end;
     result:=countout;
end;


function  VoxtoWav(nomein,nomeout : string; infreq, outfreq : integer ): String;
var
   letti,scrittitot, sspointer, xp, actualwrite: integer;
   modo : playmodes;
   fi,fo : file;
begin
     result := '';

     modo:=PropVoxType;
     {  if PropVoxType=VT_EURISTIC then
        if not euristiccheck(nomein,modo) then
         begin
         showmessage('Input file is not a dialogic kind VOX');
         exit;
         end;}
     wavehdr[6]:=outfreq;
     if PropWaveOut   then wavehdr[7]:=outfreq*2
                      else wavehdr[7]:=outfreq;
     if PropWaveOut then wavehdr[8]:=$100002
                      else wavehdr[8]:=$80001;

     sspointer:=0;
     xp:=2048;
     assignfile(fi,nomein);
     FileMode := 0;  {Set file access to read only }
     reset(fi,1);
     assignfile(fo,nomeout);
     rewrite(fo,1);
     scrittitot:=0;
     blockwrite(fo,wavehdr,sizeof(wavehdr)); { scrivo header wav }
       repeat
       blockread(fi,ADPCMbuffer,1024,letti);
       if modo=VT_ADPCM then
          begin
          AdpcmToPcm16(@ADPCMbuffer,@PCM16buffer, SSpointer, Xp, letti);
          actualwrite:=letti*4;
          end else
              begin
              case modo of
                vt_pcm : pcm8topcm16(@adpcmbuffer,@pcm16buffer,letti);
                vt_alaw: alawtopcm16(@adpcmbuffer,@pcm16buffer,letti);
                vt_mulaw: mulawtopcm16(@adpcmbuffer,@pcm16buffer,letti);
              end;
              actualwrite:=letti*2;
              end;
       if outfreq<>infreq then
         actualwrite:=2*rateconvert(@pcm16buffer,actualwrite div 2,infreq,outfreq);
   {   if force8000 then
        begin
        finalbuffer:=@pcm16buffer2;
        b:=0;
        for a:=0 to ((letti-1) div 3) do
          begin
          finalbuffer[b*4  ]:=pcm16buffer[a*3  ];
          finalbuffer[b*4+1]:=round(pcm16buffer[a*3  ]*0.25+pcm16buffer[a*3+1]*0.75);
          finalbuffer[b*4+2]:=round(pcm16buffer[a*3+1]*0.50+pcm16buffer[a*3+2]*0.50);
          finalbuffer[b*4+3]:=round(pcm16buffer[a*3+2]*0.75+pcm16buffer[a*3+3]*0.25);
          inc(b);
          end;
        actualwrite:=b*3;
        end else finalbuffer:=@pcm16buffer;  }
       if PropWaveOut then blockwrite(fo,pcm16buffer,actualwrite)
                       else
                         begin
                         actualwrite:=actualwrite div 2;
                         PCM16toPCM8(@pcm16buffer,@pcm8buffer,actualwrite);
                         blockwrite(fo,PCM8buffer,actualwrite);
                         end;
       inc(Scrittitot,actualwrite);
       until letti<>1024;

     seek(fo,0);
     wavehdr[1]:=scrittitot+36;
     wavehdr[10]:=scrittitot;
     blockwrite(fo,wavehdr,sizeof(wavehdr)); { scrivo header wav }
     closefile(fi);
     closefile(fo);
end;

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// Initialization  (MUST BE AT END OF CODE, before finalization)
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

initialization
begin
    PropVoxType           :=VT_adpcm;
    PropStoredMode        :=VT_adpcm;
    PropWaveOut           :=true;
    PropSampleDownFilter  :=true;
end;


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// finalization  (MUST BE AT END OF CODE)
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

finalization
begin
     // Nothing to do
end;

end.


