unit mySQLWorkUnit;

interface

uses
  Messages, SysUtils, MySQLDataset, DB, MySQLServer, CalcWorkUnit, TCPIPWorkUnit;

  function mySQLOpenQuery(pmySQLServer: TMySQLServer; pmySQLQuery: TMySQLQuery; pLogIfLongerThanMs: Comp; pNetHubHost: String; pNetHubPort: Integer; var pFailMessage: String): Boolean;
  procedure LogSQLTimeAndStatement(pNetHubHost: String; pNetHubPort: Integer; pLogIfLongerThanMs, pTime: Comp; pStatement: String);


implementation

(*
     // How to manage timeouts for mySQL servers
     MySQLQuery1.Close;
     GlobSQLStatement.Clear;
     GlobSQLStatement.Text :=
          trim('select                        ')+cCrLf+
          trim('  empnum,                     ')+cCrLf+
          trim('  fullname,                   ')+cCrLf+
          trim('  dept_num,                   ')+cCrLf+
          trim('  division                    ')+cCrLf+
          trim('from                          ')+cCrLf+
          trim('  employee                    ')+cCrLf+
          '';
     MySQLQuery1.SQL.Clear;
     MySQLQuery1.SQL := GlobSQLStatement;

     // Connect to the system
     if not mySQLOpenQuery(mySQLServer1, mySQLQuery1, 3000, GlobNetConIP, GlobNetConPort, mText) then
     begin
          // We got a problem.
          MsgError(mText);
     end
     else
     begin
          while not MySQLQuery1.EOF do ..... 

*)     

// Opens a Query, and will try to reconnect if there is an error
function mySQLOpenQuery(pmySQLServer: TMySQLServer; pmySQLQuery: TMySQLQuery; pLogIfLongerThanMs: Comp; pNetHubHost: String; pNetHubPort: Integer; var pFailMessage: String): Boolean;
var
   mCount :  Integer;
begin
     mCount := 0;


     // Test time to Run....
     FCTimeReportStart;    // Sets up defaults for a new report
     FCTimeCountGo;        // Start counting milliseconds again


     while true do
     begin
        try
           if not pmySQLServer.connected then
           begin
                pmySQLServer.Connected := True;
           end;

           // Run the query
           pmySQLQuery.open;

           // Log what we got
           FCTimeCountStop;      // Stop counting (this provides the total of ms)
           LogSQLTimeAndStatement(pNetHubHost, pNetHubPort, pLogIfLongerThanMs, TTTimeCount, StripOutCRLF(pmySQLQuery.SQL.Text));


           result := True;
           break;

        except
           on E: Exception do
           begin
                pFailMessage := StripOutCRLF(E.Message+'  '+IntToStr(E.HelpContext));
                // Log the error
                LogSQLTimeAndStatement(pNetHubHost, pNetHubPort, pLogIfLongerThanMs, 0, 'ERROR: '+pFailMessage+': '+StripOutCRLF(pmySQLQuery.SQL.Text));

                inc(mCount);

                if mCount <= 3 then
                begin
                     pmySQLServer.Connected := False;
                     sleep(1000);
                     continue;
                end
                else
                begin
                     // Log the error
                     LogSQLTimeAndStatement(pNetHubHost, pNetHubPort, pLogIfLongerThanMs, 0, 'FATAL: Connection Attempts failed');

                     result       := False;
                     break;
                end;
           end;

        end;
     end;
end; // mySQLOpenQuery


// Log SQL statements that are taking a very long time.
procedure LogSQLTimeAndStatement(pNetHubHost: String; pNetHubPort: Integer; pLogIfLongerThanMs, pTime: Comp; pStatement: String);
begin
     // Do something
     if (pNetHubHost <> '') and
        (pTime > pLogIfLongerThanMs) then
     begin
          HTTPLog(pNetHubHost,
                  pNetHubPort,
                  'SQLStatement('+CompToStr(pTime)+')',
                  pStatement);

     end;
end; // LogSQLTimeAndStatement

end.

